@@ -1,21 +1,32 @@
 #!/usr/bin/python3 -u
 
+# Imports from standard library
+import enum
 import os
 import signal
 import sys
-import xml.etree.ElementTree as ET
+import json
 import warnings
+from pathlib import Path
+
+sys.path.insert(1, str(Path(__file__).parents[1] / 'shared'))
+
+# imports from shared
+import osc_paths.ray as r
 
 # import subprocess and osc_server (local file) conditionnally
 # in order to answer faster in many cases.
 
-OPERATION_TYPE_NULL = 0
-OPERATION_TYPE_CONTROL = 1
-OPERATION_TYPE_SERVER = 2
-OPERATION_TYPE_SESSION = 3
-OPERATION_TYPE_CLIENT = 4
-OPERATION_TYPE_TRASHED_CLIENT = 5
-OPERATION_TYPE_ALL = 6 # for help message
+
+class OperationType(enum.Enum):
+    NULL = 0
+    CONTROL = 1
+    SERVER = 2
+    SESSION = 3
+    CLIENT = 4
+    TRASHED_CLIENT = 5
+    ALL = 6
+    'used only for help-all message'
 
 
 class Daemon:
@@ -65,7 +76,11 @@ SERVER_OPERATIONS = (
     'has_option',
     'script_info',
     'hide_script_info',
-    'script_user_action')
+    'script_user_action',
+    'auto_export_custom_names',
+    'export_custom_names',
+    'import_pretty_names',
+    'clear_pretty_names')
 
 
 def signal_handler(sig, frame):
@@ -81,7 +96,7 @@ def add_self_bin_to_path():
     if not os.environ['PATH'].startswith("%s:" % bin_path):
         os.environ['PATH'] = "%s:%s" % (bin_path, os.environ['PATH'])
 
-def pid_exists(pid: int) -> bool:
+def pid_exists(pid: int | str) -> bool:
     if isinstance(pid, str):
         pid = int(pid)
 
@@ -93,7 +108,7 @@ def pid_exists(pid: int) -> bool:
         return True
 
 def pid_is_stopped(pid: int) -> bool:
-    proc_file_path = '/proc/%i/status' % pid
+    proc_file_path = f'/proc/{pid}/status'
     if os.path.exists(proc_file_path):
         proc_file = open(proc_file_path)
         for line in proc_file.readlines():
@@ -106,49 +121,52 @@ def pid_is_stopped(pid: int) -> bool:
 
 def get_daemon_list() -> list[Daemon]:
     try:
-        tree = ET.parse('/tmp/RaySession/multi-daemon.xml')
+        with open('/tmp/RaySession/multi-daemon.json') as f:
+            json_list = json.load(f)
+    
     except:
-        return []
-
+        return list[Daemon]()
+    
+    if not isinstance(json_list, list):
+        return list[Daemon]()
+    
     l_daemon_list = list[Daemon]()
 
-    root = tree.getroot()
-    for child in root:
+    for dmn in json_list:
+        if not isinstance(dmn, dict):
+            continue
+        
         l_daemon = Daemon()
-
-        for key in child.attrib.keys():
-            if key == 'root':
-                l_daemon.root = child.attrib[key]
-            elif key == 'session_path':
-                l_daemon.session_path = child.attrib[key]
-            elif key == 'user':
-                l_daemon.user = child.attrib[key]
-            elif key == 'not_default':
-                l_daemon.not_default = bool(child.attrib[key] == '1')
-            elif key == 'net_daemon_id':
-                net_daemon_id = child.attrib[key]
-                if net_daemon_id.isdigit():
-                   l_daemon.net_daemon_id = int(net_daemon_id)
-
-            elif key == 'pid':
-                pid = child.attrib[key]
-                if pid.isdigit() and pid_exists(pid):
-                    l_daemon.pid = int(pid)
-
-            elif key == 'port':
-                l_port = child.attrib[key]
-                if l_port.isdigit():
-                    l_daemon.port = int(l_port)
-
-            elif key == 'has_gui':
-                l_daemon.has_local_gui = bool(child.attrib[key] == '3')
-                l_daemon.has_gui = bool(child.attrib[key] == '1')
-                
-            elif key == 'local_gui_pids':
-                gui_pids_str = child.attrib[key]
-                for pid_str in gui_pids_str.split(':'):
-                    if pid_str.isdigit():
-                        l_daemon.local_gui_pids.append(int(pid_str))
+        
+        for key, value in dmn.items():
+            match key:
+                case 'root':
+                    l_daemon.root = str(value)
+                case 'session_path':
+                    l_daemon.session_path = str(value)
+                case 'user':
+                    l_daemon.user = str(value)
+                case 'not_default':
+                    l_daemon.not_default = bool(value)
+                case 'net_daemon_id':
+                    if isinstance(value, int):
+                        l_daemon.net_daemon_id = int(value)
+                case 'pid':
+                    if isinstance(value, int) and pid_exists(value):
+                        l_daemon.pid = value
+                case 'port':
+                    if isinstance(value, int):
+                        l_daemon.port = value
+                case 'has_gui':
+                    if isinstance(value, int):
+                        l_daemon.has_gui = bool(value == 1)
+                        l_daemon.has_local_gui = bool(value == 3)
+                    
+                case 'local_gui_pids':
+                    if isinstance(value, list):
+                        for pid in value:
+                            if isinstance(pid, int):
+                                l_daemon.local_gui_pids.append(pid)
 
         if not (l_daemon.net_daemon_id
                 and l_daemon.pid
@@ -158,18 +176,16 @@ def get_daemon_list() -> list[Daemon]:
         l_daemon_list.append(l_daemon)
     return l_daemon_list
 
-def print_help(stdout=False, category=OPERATION_TYPE_NULL):
-    script_dir = os.path.dirname(os.path.realpath(sys.argv[0]))
+def print_help(stdout=False, category=OperationType.NULL):
     lang_file = "help_en_US"
-
-    if os.getenv('LANG').startswith('fr_'):
+    if os.getenv('LANG', '').startswith('fr_'):
         lang_file = "help_fr_FR"
 
-    help_path = "%s/%s" % (script_dir, lang_file)
+    help_path = Path(__file__).parent / lang_file
 
     try:
-        help_file = open(help_path, 'r')
-        full_message = help_file.read()
+        with open(help_path, 'r') as help_file:
+            full_message = help_file.read()
     except:
         sys.stderr.write('error: help_file %s is missing\n' % help_path)
         sys.exit(101)
@@ -177,7 +193,7 @@ def print_help(stdout=False, category=OPERATION_TYPE_NULL):
     message = ''
     stars = 0
 
-    if category == OPERATION_TYPE_ALL:
+    if category is OperationType.ALL:
         message = full_message
     else:
         for line in full_message.split('\n'):
@@ -185,10 +201,10 @@ def print_help(stdout=False, category=OPERATION_TYPE_NULL):
                 stars += 1
 
             if (stars == 0
-                    or (stars == 1 and category == OPERATION_TYPE_CONTROL)
-                    or (stars == 2 and category == OPERATION_TYPE_SERVER)
-                    or (stars == 3 and category == OPERATION_TYPE_SESSION)
-                    or (stars >= 4 and category == OPERATION_TYPE_CLIENT)):
+                    or (stars == 1 and category is OperationType.CONTROL)
+                    or (stars == 2 and category is OperationType.SERVER)
+                    or (stars == 3 and category is OperationType.SESSION)
+                    or (stars >= 4 and category is OperationType.CLIENT)):
                 message += "%s\n" % line
 
     if stdout:
@@ -205,7 +221,8 @@ def auto_type_string(string: str):
     return string
 
 
-if __name__ == '__main__':
+# if __name__ == '__main__':
+if True:
     warnings.filterwarnings("ignore", category=DeprecationWarning)
     add_self_bin_to_path()
 
@@ -214,7 +231,7 @@ if __name__ == '__main__':
         sys.exit(100)
 
     terminate = False
-    operation_type = OPERATION_TYPE_NULL
+    operation_type = OperationType.NULL
     client_id = ''
 
     args = sys.argv[1:]
@@ -229,65 +246,68 @@ if __name__ == '__main__':
     while args and args[0].startswith('--'):
         option = args.pop(0)
 
-        if option.startswith('--help'):
-            if option == '--help':
-                print_help(True, OPERATION_TYPE_NULL)
-            elif option == '--help-all':
-                print_help(True, OPERATION_TYPE_ALL)
-            elif option == '--help-control':
-                print_help(True, OPERATION_TYPE_CONTROL)
-            elif option == '--help-server':
-                print_help(True, OPERATION_TYPE_SERVER)
-            elif option == '--help-session':
-                print_help(True, OPERATION_TYPE_SESSION)
-            elif option in ('--help-client', '--help-clients'):
-                print_help(True, OPERATION_TYPE_CLIENT)
-            else:
+        match option:
+            case opt if opt.startswith('--help'):
+                match opt:
+                    case '--help':
+                        print_help(True, OperationType.NULL)
+                    case '--help-all':
+                        print_help(True, OperationType.ALL)
+                    case '--help-control':
+                        print_help(True, OperationType.CONTROL)
+                    case '--help-server':
+                        print_help(True, OperationType.SERVER)
+                    case '--help-session':
+                        print_help(True, OperationType.SESSION)
+                    case '--help-client' | '--help-clients':
+                        print_help(True, OperationType.CLIENT)
+                    case _:
+                        print_help()
+                        sys.exit(100)
+
+                sys.exit(0)
+
+            case '--port':
+                if not args:
+                    print_help()
+                    sys.exit(100)
+                port = args.pop(0)
+                if not port.isdigit():
+                    sys.stderr.write('Invalid value for port: %s . Use digits !'
+                                    % port)
+                    sys.exit(100)
+                wanted_port = int(port)
+
+            case '--detach':
+                detach = True
+            case _:
                 print_help()
                 sys.exit(100)
-            sys.exit(0)
-
-        elif option == '--port':
-            if not args:
-                print_help()
-                sys.exit(100)
-            port = args.pop(0)
-            if not port.isdigit():
-                sys.stderr.write('Invalid value for port: %s . Use digits !'
-                                 % port)
-                sys.exit(100)
-            wanted_port = int(port)
-
-        elif option == '--detach':
-            detach = True
-        else:
-            print_help()
-            sys.exit(100)
 
     operation = args.pop(0)
     if operation in ('client', 'trashed_client'):
         if len(args) < 2:
-            print_help(False, OPERATION_TYPE_CLIENT)
+            print_help(False, OperationType.CLIENT)
             sys.exit(100)
 
-        operation_type = OPERATION_TYPE_CLIENT
+        operation_type = OperationType.CLIENT
         if operation == 'trashed_client':
-            operation_type = OPERATION_TYPE_TRASHED_CLIENT
+            operation_type = OperationType.TRASHED_CLIENT
 
         client_id = args.pop(0)
         operation = args.pop(0)
 
-    if not operation_type:
+    if operation_type is OperationType.NULL:
         if operation in CONTROL_OPERATIONS:
-            operation_type = OPERATION_TYPE_CONTROL
+            operation_type = OperationType.CONTROL
         elif operation in SERVER_OPERATIONS:
-            operation_type = OPERATION_TYPE_SERVER
+            operation_type = OperationType.SERVER
         else:
-            operation_type = OPERATION_TYPE_SESSION
+            operation_type = OperationType.SESSION
 
     arg_list = [auto_type_string(s) for s in args]
-    if operation_type in (OPERATION_TYPE_CLIENT,
-                          OPERATION_TYPE_TRASHED_CLIENT):
+    if operation_type in (OperationType.CLIENT,
+                          OperationType.TRASHED_CLIENT):
         arg_list.insert(0, client_id)
 
     if operation in ('new_session', 'open_session', 'change_root',
@@ -314,44 +334,49 @@ if __name__ == '__main__':
     else:
         daemon_started = False
 
-    if operation_type == OPERATION_TYPE_CONTROL:
-        if operation == 'start':
-            if daemon_started:
-                sys.stderr.write('server already started.\n')
+    if operation_type is OperationType.CONTROL:
+        match operation:
+            
+            # global operations
+
+            case 'start':
+                if daemon_started:
+                    sys.stderr.write('server already started.\n')
+                    sys.exit(0)
+
+            case 'start_new' | 'start_new_hidden':
+                pass
+
+            case 'stop':
+                if not daemon_started:
+                    sys.stderr.write('No server started.\n')
+                    sys.exit(0)
+
+            case 'list_daemons':
+                for daemon in daemon_list:
+                    if daemon.not_default:
+                        continue
+                    sys.stdout.write('%s\n' % str(daemon.port))
                 sys.exit(0)
 
-        elif operation in ('start_new', 'start_new_hidden'):
-            pass
-
-        elif operation == 'stop':
-            if not daemon_started:
-                sys.stderr.write('No server started.\n')
-                sys.exit(0)
-
-        elif operation == 'list_daemons':
-            for daemon in daemon_list:
-                if daemon.not_default:
-                    continue
-                sys.stdout.write('%s\n' % str(daemon.port))
-            sys.exit(0)
-
-        else:
-            if not daemon_started:
+            case _ if not daemon_started:
                 sys.stderr.write(
                     'No server started. So impossible to %s\n' % operation)
                 sys.exit(100)
 
-            if operation == 'get_pid':
+            # operations needing a running daemon
+
+            case 'get_pid':
                 for daemon in daemon_list:
                     if daemon.port == daemon_port:
                         sys.stdout.write('%s\n' % str(daemon.pid))
                         sys.exit(0)
 
-            elif operation == 'get_port':
+            case 'get_port':
                 sys.stdout.write("%s\n" % str(daemon_port))
                 sys.exit(0)
 
-            elif operation == 'get_port_gui_free':
+            case 'get_port_gui_free':
                 wanted_session_root = ''
                 if args:
                     wanted_session_root = args[0]
@@ -359,7 +384,7 @@ if __name__ == '__main__':
                 for daemon in daemon_list:
                     if (daemon.user == os.environ['USER']
                             and (daemon.root == wanted_session_root
-                                 or not wanted_session_root)
+                                or not wanted_session_root)
                             and not daemon.not_default):
                         if not daemon.has_local_gui:
                             sys.stdout.write('%s\n' % daemon.port)
@@ -379,13 +404,13 @@ if __name__ == '__main__':
 
                 sys.exit(0)
 
-            elif operation == 'get_root':
+            case 'get_root':
                 for daemon in daemon_list:
                     if daemon.port == daemon_port:
                         sys.stdout.write('%s\n' % daemon.root)
                         sys.exit(0)
 
-            elif operation == 'get_session_path':
+            case 'get_session_path':
                 for daemon in daemon_list:
                     if daemon.port == daemon_port:
                         if not daemon.session_path:
@@ -393,14 +418,14 @@ if __name__ == '__main__':
                         sys.stdout.write('%s\n' % daemon.session_path)
                         sys.exit(0)
 
-            elif operation == 'has_local_gui':
+            case 'has_local_gui':
                 for daemon in daemon_list:
                     if daemon.port == daemon_port:
                         if daemon.has_local_gui:
                             sys.exit(0)
                 sys.exit(1)
 
-            elif operation == 'has_gui':
+            case 'has_gui':
                 for daemon in daemon_list:
                     if daemon.port == daemon_port:
                         if daemon.has_gui:
@@ -412,54 +437,58 @@ if __name__ == '__main__':
         if daemon_port:
             at_port = "at port %i" % daemon_port
 
-        if operation_type == OPERATION_TYPE_SERVER:
-            if operation == 'quit':
-                sys.stderr.write('No server %s to quit !\n' % at_port)
-                sys.exit(0)
+        match operation_type:
+            case OperationType.SERVER:
+                if operation == 'quit':
+                    sys.stderr.write(f'No server {at_port} to quit !\n')
+                    sys.exit(0)
 
-        elif operation_type == OPERATION_TYPE_SESSION:
-            sys.stderr.write("No server started %s. So no session to %s\n"
-                                 % (at_port, operation))
-            sys.exit(100)
-        elif operation_type == OPERATION_TYPE_CLIENT:
-            sys.stderr.write("No server started %s. So no client to %s\n"
-                                 % (at_port, operation))
-            sys.exit(100)
-        elif operation_type == OPERATION_TYPE_CLIENT:
-            sys.stderr.write(
-                "No server started %s. So no trashed client to %s\n"
-                                 % (at_port, operation))
-            sys.exit(100)
-        else:
-            print_help()
-            sys.exit(100)
+            case OperationType.SESSION:
+                sys.stderr.write(
+                    f"No server started {at_port}. "
+                    f"So no session to {operation}\n")
+                sys.exit(100)
+            case OperationType.CLIENT:
+                sys.stderr.write(
+                    f"No server started {at_port}. "
+                    f"So no client to {operation}\n")
+                sys.exit(100)
+            case OperationType.TRASHED_CLIENT:
+                sys.stderr.write(
+                    f"No server started {at_port}. "
+                    f"So no trashed client to {operation}\n")
+                sys.exit(100)
+            case _:
+                print_help()
+                sys.exit(100)
 
     osc_order_path = '/ray/'
-    if operation_type == OPERATION_TYPE_CLIENT:
-        osc_order_path += 'client/'
-    elif operation_type == OPERATION_TYPE_TRASHED_CLIENT:
-        osc_order_path += 'trashed_client/'
-    elif operation_type == OPERATION_TYPE_SERVER:
-        osc_order_path += 'server/'
-    elif operation_type == OPERATION_TYPE_SESSION:
-        osc_order_path += 'session/'
+    match operation_type:
+        case OperationType.CLIENT:
+            osc_order_path += 'client/'
+        case OperationType.TRASHED_CLIENT:
+            osc_order_path += 'trashed_client/'
+        case OperationType.SERVER:
+            osc_order_path += 'server/'
+        case OperationType.SESSION:
+            osc_order_path += 'session/'
 
     osc_order_path += operation
 
-    if operation_type == OPERATION_TYPE_CONTROL and operation == 'stop':
-        osc_order_path = '/ray/server/quit'
+    if operation_type is OperationType.CONTROL and operation == 'stop':
+        osc_order_path = r.server.QUIT
 
-    import osc_server  # see top of the file    
-    server = osc_server.OscServer(detach)
+    from osc_server import OscServer  # see top of the file
+    server = OscServer(detach)
     server.set_order_path_args(osc_order_path, arg_list)
     daemon_process = None
 
     if (daemon_started
-            and not (operation_type == OPERATION_TYPE_CONTROL
+            and not (operation_type is OperationType.CONTROL
                      and operation in ('start_new', 'start_new_hidden'))):
-        if (operation_type == OPERATION_TYPE_CONTROL
+        if (operation_type is OperationType.CONTROL
                 and operation == 'stop'):
-            daemon_port_list = []
+            daemon_port_list = list[int]()
 
             if wanted_port:
                 daemon_port_list.append(wanted_port)
@@ -498,18 +527,18 @@ if __name__ == '__main__':
             process_args.append('--osc-port')
             process_args.append(str(wanted_port))
 
-        if (operation_type == OPERATION_TYPE_CONTROL
+        if (operation_type is OperationType.CONTROL
                 and operation == 'start_new_hidden'):
             process_args.append('--hidden')
             process_args.append('--no-options')
 
-        daemon_process = subprocess.Popen(process_args, -1, None, None,
-                                          subprocess.DEVNULL,
-                                          subprocess.DEVNULL)
+        daemon_process = subprocess.Popen(
+            process_args, -1, None, None,
+            subprocess.DEVNULL, subprocess.DEVNULL)
 
         server.wait_for_start()
 
-        if (operation_type == OPERATION_TYPE_CONTROL
+        if (operation_type is OperationType.CONTROL
                 and operation in ('start', 'start_new', 'start_new_hidden')):
             server.wait_for_start_only()
 
@@ -533,12 +562,12 @@ if __name__ == '__main__':
             exit_code = 103
             break
 
-        if daemon_process and not daemon_process.poll() is None:
+        if daemon_process and daemon_process.poll() is not None:
             sys.stderr.write('daemon terminates, sorry\n')
             exit_code = 104
             break
 
-    if (operation_type == OPERATION_TYPE_CONTROL
+    if (operation_type is OperationType.CONTROL
             and operation in ('start_new', 'start_new_hidden')
             and exit_code == 0):
         daemon_port = server.get_daemon_port()
