@@ -821,7 +821,7 @@ class Client(ServerSender, ray.ClientData):
 
         return jack_client_name
 
-    def read_xml_properties(self, c: XmlElement, old_mode=False):
+    def read_xml_properties(self, c: XmlElement):
         self.executable_path = c.string('executable')
         self.arguments = c.string('arguments')
         self.pre_env = c.string('pre_env')
@@ -836,9 +836,8 @@ class Client(ServerSender, ray.ClientData):
         self.start_gui_hidden = not c.bool('gui_visible', True)
         self.template_origin = c.string('template_origin')
 
-        if old_mode:
-            self.jack_naming = ray.JackNaming.SHORT
-            self.prefix_mode = ray.PrefixMode.SESSION_NAME
+        self.jack_naming = ray.JackNaming.SHORT
+        self.prefix_mode = ray.PrefixMode.SESSION_NAME
 
         if c.string('jack_naming'):
             self.jack_naming = ray.JackNaming(int(c.bool('jack_naming')))
