@@ -1,18 +1,24 @@
 #!/usr/bin/python3 -u
 
+# Imports from standard library
 import os
 import subprocess
 from typing import TYPE_CHECKING
 import warnings
 
-from PyQt5.QtCore import QProcess
+# third party imports
+from qtpy.QtCore import QProcess
 
+# Imports from src/shared
 from xml_tools import XmlElement
+
+# Local imports
 from daemon_tools import is_pid_child_of
 
 if TYPE_CHECKING:
     from session import Session
 
+
 def move_win(win_id, desktop_from, desktop_to):
     if desktop_from == desktop_to:
         return
@@ -186,9 +192,9 @@ class DesktopsMemory:
                 continue
             
             win = WindowProperties()
-            win.wclass = w.str('class')
-            win.name = w.str('name')
-            desktop = w.str('desktop')
+            win.wclass = w.string('class')
+            win.name = w.string('name')
+            desktop = w.string('desktop')
             if desktop.lstrip('-').isdigit():
                 win.desktop = int(desktop)
             
