@@ -440,9 +440,6 @@ class OperatingSession(Session):
             self.load_error(ray.Err.BAD_PROJECT)
             return
         
-        sess_version = root.attrib.get('VERSION', '0.8.0')
-        old_mode = bool(ray.version_to_tuple(sess_version) < (0, 17, 0))
-        
         root.attrib['name'] = spath.name
 
         tmp_clients = list[Client]()
@@ -453,8 +450,7 @@ class OperatingSession(Session):
 
             for client_xml in child:
                 client = Client(self)
-                client.read_xml_properties(
-                    XmlElement(client_xml), old_mode=old_mode)
+                client.read_xml_properties(XmlElement(client_xml))
                 if not client.executable_path:
                     continue
             
@@ -1266,8 +1262,6 @@ for better organization.""")
 
             xroot = XmlElement(root)
             sess_name = xroot.string('name')
-            sess_version = xroot.string('VERSION', '0.9.0')
-            old_mode = ray.version_to_tuple(sess_version) < (0, 17, 0)
             
             if xroot.bool('notes_shown'):
                 self.future_notes_shown = True
@@ -1279,7 +1273,7 @@ for better organization.""")
                     for cchild in child:
                         c = XmlElement(cchild)
                         client = Client(self)
-                        client.read_xml_properties(c, old_mode=old_mode)
+                        client.read_xml_properties(c)
                         
                         if not client.executable_path:
                             continue
