@@ -1,22 +1,33 @@
 
+# Imports from standard library
 import os
 import socket
 import logging
 from pathlib import Path
 from typing import TYPE_CHECKING, Callable, Optional
-from PyQt5.QtCore import QProcess, QObject, QDateTime
-from PyQt5.QtXml import QDomDocument, QDomElement
 import xml.etree.ElementTree as ET
+from xml.etree.ElementTree import Element
 
+# third party imports
+from qtpy.QtCore import QProcess, QObject, QDateTime
+
+# Imports from src/shared
 import ray
-from daemon_tools import Terminal
 from xml_tools import XmlElement
+import osc_paths
+import osc_paths.ray as r
+import osc_paths.ray.gui as rg
+
+# Local imports
+from daemon_tools import Terminal
 
 if TYPE_CHECKING:
     from session import Session
-    
+
+
 _logger = logging.getLogger(__name__)
 
+
 def git_stringer(string:str) -> str:
     for char in (' ', '*', '?', '[', ']', '(', ')'):
         string = string.replace(char, "\\" + char)
@@ -82,7 +93,7 @@ class Snapshoter(QObject):
 
         self._n_file_treated += len(standard_output.splitlines()) -1
 
-        self.session.send_gui('/ray/gui/server/progress',
+        self.session.send_gui(rg.server.PROGRESS,
                               self._n_file_treated / self._n_file_changed)
 
     def _standard_error(self):
@@ -93,15 +104,16 @@ class Snapshoter(QObject):
         standard_output = self._git_process.readAllStandardOutput().data()
         Terminal.snapshoter_message(standard_output, self._git_command)
 
-    def _run_git_process(self, *all_args) -> int:
-        return self._run_git_process_at(self.session.path, *all_args)
+    def _run_git_process(self, *all_args) -> bool:
+        return self._run_git_process_at(str(self.session.path), *all_args)
 
-    def _run_git_process_at(self, spath, *all_args) -> int:
+    def _run_git_process_at(self, spath: str, *all_args) -> bool:
         self._git_command = ''
         for arg in all_args:
             self._git_command += ' %s' % arg
 
         err = ray.Err.OK
+        exit_code = 0
 
         git_args = self._get_git_command_list_at(spath, *all_args)
         self._git_process.start(self._git_exec, git_args)
@@ -109,49 +121,48 @@ class Snapshoter(QObject):
             self._git_process.kill()
             err = ray.Err.SUBPROCESS_UNTERMINATED
         else:
-            if self._git_process.exitStatus():
+            exit_code = self._git_process.exitCode()
+            if self._git_process.exitStatus() == QProcess.ExitStatus.CrashExit:
                 err = ray.Err.SUBPROCESS_CRASH
-            elif self._git_process.exitCode():
+            elif exit_code:
                 err = ray.Err.SUBPROCESS_EXITCODE
 
         if err and self._error_function:
-            self._error_function(err, ' '.join(all_args))
+            self._error_function(err, ' '.join(all_args), exit_code)
 
         return not bool(err)
 
     def _get_git_command_list(self, *args) -> list[str]:
-        return self._get_git_command_list_at(self.session.path, *args)
+        return self._get_git_command_list_at(str(self.session.path), *args)
 
     def _get_git_command_list_at(self, spath: str, *args) -> list[str]:
         first_args = ['--work-tree', spath, '--git-dir',
-                      "%s/%s" % (spath, self._gitdir)]
-
+                      str(Path(spath) / self._gitdir)]
         return first_args + list(args)
 
-    def _get_history_full_path(self):
-        return "%s/%s/%s" % (
-                        self.session.path, self._gitdir, self._history_path)
-
-    def _get_history_xml_document_element(self) -> Optional[QDomElement]:
+    def _get_history_full_path(self) -> Path:
+        return self.session.path / self._gitdir / self._history_path
+    
+    def _get_history_xml_root(self) -> Optional[Element]:
         if not self._is_init():
             return None
-
+        
         file_path = self._get_history_full_path()
-
-        xml = QDomDocument()
-
+        if not file_path.exists():
+            return None
+        
         try:
-            history_file = open(file_path, 'r')
-            xml.setContent(history_file.read())
-            history_file.close()
-        except BaseException:
+            with open(file_path, 'r') as file:
+                tree = ET.parse(file)
+        except BaseException as e:
+            _logger.error(f'Failed to parse {file_path} as an XML file')
+            _logger.error(str(e))
             return None
-
-        SNS_xml = xml.documentElement()
-        if SNS_xml.tagName() != 'SNAPSHOTS':
+        
+        root = tree.getroot()
+        if root.tag != 'SNAPSHOTS':
             return None
-
-        return SNS_xml
+        return root            
 
     def _get_tag_date(self)->str:
         date_time = QDateTime.currentDateTimeUtc()
@@ -164,8 +175,9 @@ class Snapshoter(QObject):
 
         return tagdate
 
-    def _write_history_file(self, date_str: str, snapshot_name='', rewind_snapshot='') -> int:
-        if not self.session.path:
+    def _write_history_file(
+            self, date_str: str, snapshot_name='', rewind_snapshot='') -> int:
+        if self.session.path is None:
             return ray.Err.NO_SESSION_OPEN
 
         file_path = self._get_history_full_path()
@@ -195,39 +207,34 @@ class Snapshoter(QObject):
             
             for client_file_path in client.get_project_files():
                 base_path = str(
-                    Path(client_file_path).relative_to(self.session.path))
+                    client_file_path.relative_to(self.session.path))
                 file_xml = ET.SubElement(client_el, 'file')
                 fxl = XmlElement(file_xml)
                 fxl.set_str('path', base_path)
                 
         try:
-            tree.write(file_path)
+            tree.write(str(file_path))
             return ray.Err.OK
         except BaseException as e:
             _logger.error(str(e))
             return ray.Err.CREATE_FAILED
 
-    def _get_exclude_file_full_path(self) -> str:
-        return "%s/%s/%s" % (
-                        self.session.path, self._gitdir, self._exclude_path)
+    def _write_exclude_file(self) -> ray.Err:
+        if self.session.path is None:
+            return ray.Err.NO_SESSION_OPEN
+        
+        file_path = self.session.path / self._gitdir / self._exclude_path
 
-    def _write_exclude_file(self) -> int:
-        file_path = self._get_exclude_file_full_path()
-
-        try:
-            exclude_file = open(file_path, 'w')
-        except:
-            return ray.Err.CREATE_FAILED
-
-        contents = ""
-        contents += "# This file is generated by ray-daemon at each snapshot\n"
-        contents += "# Don't edit this file.\n"
-        contents += "# If you want to add/remove files managed by git\n"
-        contents += "# Create/Edit .gitignore in the session folder\n"
-        contents += "\n"
-        contents += "%s\n" % self._gitdir
-        contents += "\n"
-        contents += "# Globally ignored extensions\n"
+        contents = (
+            "# This file is generated by ray-daemon at each snapshot\n"
+            "# Don't edit this file.\n"
+            "# If you want to add/remove files managed by git\n"
+            "# Create/Edit .gitignore in the session folder\n"
+            "\n"
+            f"{self._gitdir}\n"
+            "\n"
+            "# Globally ignored extensions\n"
+        )
 
         session_ignored_extensions = ray.GIT_IGNORED_EXTENSIONS
         session_ign_list = session_ignored_extensions.split(' ')
@@ -242,11 +249,11 @@ class Snapshoter(QObject):
 
                 if not extension in cext_list:
                     contents += "!%s.%s/**/*%s\n" % (
-                        git_stringer(client.get_prefix_string()),
+                        git_stringer(client.prefix),
                         git_stringer(client.client_id),
                         extension)
                     contents += "!%s.%s.**/*%s\n" % (
-                        git_stringer(client.get_prefix_string()),
+                        git_stringer(client.prefix),
                         git_stringer(client.client_id),
                         extension)
 
@@ -264,12 +271,12 @@ class Snapshoter(QObject):
                     continue
 
                 contents += "%s.%s/**/*%s\n" % (
-                    git_stringer(client.get_prefix_string()),
+                    git_stringer(client.prefix),
                     git_stringer(client.client_id),
                     extension)
 
                 contents += "%s.%s.**/*%s\n" % (
-                    git_stringer(client.get_prefix_string()),
+                    git_stringer(client.prefix),
                     git_stringer(client.client_id),
                     extension)
 
@@ -281,14 +288,13 @@ class Snapshoter(QObject):
         for foldername, subfolders, filenames in os.walk(self.session.path):
             subfolders[:] = [d for d in subfolders if d not in no_check_list]
 
-            if foldername == "%s/%s" % (self.session.path, self._gitdir):
+            if foldername == str(self.session.path / self._gitdir):
                 continue
 
             for filename in filenames:
                 if filename.endswith(session_ign_list):
                     if os.path.islink(filename):
-                        short_folder = foldername.replace(
-                                        self.session.path + '/', '', 1)
+                        short_folder = Path(foldername).relative_to(self.session.path)
                         line = git_stringer("%s/%s" % (short_folder, filename))
                         contents += '!%s\n' % line
                     # file with extension globally ignored but
@@ -305,33 +311,33 @@ class Snapshoter(QObject):
                 except:
                     continue
 
-                if file_size > self._max_file_size*1024**2:
-                    if foldername == self.session.path:
+                if file_size > self._max_file_size * 1024 ** 2:
+                    if foldername == str(self.session.path):
                         line = git_stringer(filename)
                     else:
-                        short_folder = foldername.replace(
-                                        self.session.path + '/', '', 1)
+                        short_folder = Path(foldername).relative_to(
+                            self.session.path)
                         line = git_stringer("%s/%s" % (short_folder, filename))
 
                     contents += "%s\n" % line
 
         try:
-            exclude_file.write(contents)
-            exclude_file.close()
+            with open(file_path, 'w') as exclude_file:
+                exclude_file.write(contents)
         except:
             return ray.Err.CREATE_FAILED
 
         return ray.Err.OK
 
     def _is_init(self) -> bool:
-        if not self.session.path:
+        if self.session.path is None:
             return False
 
-        return os.path.isfile("%s/%s/%s" % (
-                self.session.path, self._gitdir, self._exclude_path))
+        exclude_file = self.session.path / self._gitdir / self._exclude_path
+        return exclude_file.is_file()
 
     def _can_save(self) -> bool:
-        if not self.session.path:
+        if self.session.path is None:
             return False
 
         if not self._is_init():
@@ -392,9 +398,12 @@ class Snapshoter(QObject):
                     self._error_function(err)
 
             # not really a reply, not strong.
-            self.session.send_gui('/reply', '/ray/session/list_snapshots',
-                                  full_ref_for_gui(ref, self._next_snapshot_name,
-                                                   self._rw_snapshot))
+            self.session.send_gui(
+                osc_paths.REPLY,
+                r.session.LIST_SNAPSHOTS,
+                full_ref_for_gui(ref, self._next_snapshot_name,
+                                 self._rw_snapshot))
+
         self._error_function = None
         self._next_snapshot_name = ''
         self._rw_snapshot = ''
@@ -403,44 +412,36 @@ class Snapshoter(QObject):
             self._next_function()
 
     def list(self, client_id="") -> list[str]:
-        SNS_xml = self._get_history_xml_document_element()
-        if not SNS_xml:
-            return []
-
-        nodes = SNS_xml.childNodes()
+        root = self._get_history_xml_root()
+        if root is None:
+            return list[str]()
 
         all_tags = list[str]()
         all_snaps = list[tuple[str, str]]()
         prv_session_name = self.session.name
-
-        for i in range(nodes.count()):
-            node = nodes.at(i)
-            el = node.toElement()
-
+        
+        for child in root:
             if client_id:
-                client_nodes = node.childNodes()
-                for j in range(client_nodes.count()):
-                    client_node = client_nodes.at(j)
-                    client_el = client_node.toElement()
-                    if client_el.attribute('client_id') == client_id:
+                for client_ch in child:
+                    if client_ch.attrib.get('client_id') == client_id:
                         break
                 else:
                     continue
-
-            ref = el.attribute('ref')
-            name = el.attribute('name')
-            rw_sn = el.attribute('rewind_snapshot')
-            rw_name = ""
-            session_name = el.attribute('session_name')
-
+            
+            ref = child.attrib.get('ref', '')
+            name = child.attrib.get('name', '')
+            rw_sn = child.attrib.get('rewind_snapshot')
+            rw_name = ''
+            session_name = child.attrib.get('session_name')
+            
             # don't list snapshot from client before session renamed
             if client_id and session_name != self.session.name:
                 client = self.session.get_client(client_id)
                 if (client
                         and (client.prefix_mode
-                             == ray.PrefixMode.SESSION_NAME)):
+                             is ray.PrefixMode.SESSION_NAME)):
                     continue
-
+                
             ss_name = ""
             if session_name != prv_session_name:
                 ss_name = session_name
@@ -461,22 +462,22 @@ class Snapshoter(QObject):
                     if snap[0] == rw_sn and not '\n' in snap[1]:
                         rw_name = snap[1]
                         break
-
+                    
             all_snaps.append((ref, name))
-            snapsss = full_ref_for_gui(ref, name, rw_sn, rw_name, ss_name)
-            all_tags.append(snapsss)
+            all_tags.append(
+                full_ref_for_gui(ref, name, rw_sn, rw_name, ss_name))
 
         all_tags.reverse()
         return all_tags
 
     def has_changes(self) -> bool:
-        if not self.session.path:
+        if self.session.path is None:
             return False
 
         if not self._is_init():
             return True
 
-        if self._changes_checker.state():
+        if self._changes_checker.state() != QProcess.ProcessState.NotRunning:
             self._changes_checker.kill()
 
         self._n_file_changed = 0
@@ -498,7 +499,7 @@ class Snapshoter(QObject):
         self._error_function = error_function
 
         if not self._can_save():
-            Terminal.message("can't snapshot")
+            self.session.message("can't snapshot")
             return
 
         err = self._write_exclude_file()
@@ -512,8 +513,6 @@ class Snapshoter(QObject):
             self.has_changes()
 
         self._changes_counted = False
-        
-        print('manikal', self._n_file_changed)
 
         if self._n_file_changed:
             all_args = self._get_git_command_list('add', '-A', '-v')
@@ -523,67 +522,55 @@ class Snapshoter(QObject):
 
         # self.adder_process.finished is connected to self._save_step_1
 
-    def load(self, spath: str, snapshot: str, error_function: Callable):
+    def load(self, spath: Path, snapshot: str, error_function: Callable):
         self._error_function = error_function
 
         snapshot_ref = snapshot.partition('\n')[0].partition(':')[0]
 
-        if not self._run_git_process_at(spath, 'reset', '--hard'):
+        if not self._run_git_process_at(str(spath), 'reset', '--hard'):
             return False
 
-        if not self._run_git_process_at(spath, 'checkout', snapshot_ref):
+        if not self._run_git_process_at(str(spath), 'checkout', snapshot_ref):
             return False
         return True
 
     def load_client_exclusive(self, client_id: str, snapshot: str,
                               error_function: Callable):
+        '''load a snapshot only for a client,
+        it will change files affected by the client'''
         self._error_function = error_function
 
-        SNS_xml = self._get_history_xml_document_element()
-        if not SNS_xml:
+        root = self._get_history_xml_root()
+        if root is None:
             self._error_function(ray.Err.NO_SUCH_FILE,
-                                self._get_history_full_path())
+                                 str(self._get_history_full_path()))
             return False
-
-        nodes = SNS_xml.childNodes()
-
+        
         client_path_list = list[str]()
-
-        for i in range(nodes.count()):
-            node = nodes.at(i)
-            el = node.toElement()
-
-            if el.attribute('ref') != snapshot:
+        
+        for child in root:
+            if child.attrib.get('ref') != snapshot:
                 continue
-
-            client_nodes = node.childNodes()
-
-            for j in range(client_nodes.count()):
-                client_node = client_nodes.at(j)
-                client_el = client_node.toElement()
-
-                if client_el.attribute('client_id') != client_id:
+            
+            for client_ch in child:
+                if client_ch.attrib.get('client_id') != client_id:
                     continue
-
-                file_nodes = client_node.childNodes()
-
-                for k in range(file_nodes.count()):
-                    file_node = file_nodes.at(k)
-                    file_el = file_node.toElement()
-                    file_path = file_el.attribute('path')
+                
+                for file_ch in client_ch:
+                    file_path = file_ch.attrib.get('path')
                     if file_path:
                         client_path_list.append(file_path)
 
         if not self._run_git_process('reset', '--hard'):
             return False
 
-        if not self._run_git_process('checkout', snapshot, '--',
-                                  *client_path_list):
+        if not self._run_git_process(
+                'checkout', snapshot, '--', *client_path_list):
             return False
         return True
 
     def abort(self):
-        if not self._adder_process.state():
+        if self._adder_process.state() == QProcess.ProcessState.NotRunning:
             return
 
         self.set_auto_snapshot(False)
@@ -591,10 +578,13 @@ class Snapshoter(QObject):
         self._adder_aborted = True
         self._adder_process.terminate()
 
-    def set_auto_snapshot(self, bool_snapshot):
-        auto_snap_file = "%s/%s/prevent_auto_snapshot" % (self.session.path,
-                                                          self._gitdir)
-        file_exists = bool(os.path.exists(auto_snap_file))
+    def set_auto_snapshot(self, bool_snapshot: bool):
+        if self.session.path is None:
+            return
+        
+        auto_snap_file = (
+            self.session.path / self._gitdir / 'prevent_auto_snapshot')
+        file_exists = auto_snap_file.exists()
 
         if bool_snapshot:
             if file_exists:
@@ -608,12 +598,16 @@ class Snapshoter(QObject):
                 contents += "# remove it if you want auto snapshots back"
 
                 try:
-                    file = open(auto_snap_file, 'w')
-                    file.write(contents)
-                    file.close()
+                    with open(auto_snap_file, 'w') as file:
+                        file.write(contents)
                 except PermissionError:
                     return
 
-    def is_auto_snapshot_prevented(self)->bool:
-        auto_snap_file = "%s/%s/prevent_auto_snapshot" % (self.session.path, self._gitdir)
-        return bool(os.path.exists(auto_snap_file))
+    def is_auto_snapshot_prevented(self) -> bool:
+        if self.session.path is None:
+            return False
+
+        auto_snap_file = (
+            self.session.path / self._gitdir / 'prevent_auto_snapshot')
+        return auto_snap_file.exists()
+        
