@@ -1,25 +1,35 @@
+
+# Imports from standard library
 from typing import TYPE_CHECKING
-from PyQt5.QtCore import Qt, QSize
-from PyQt5.QtWidgets import (QDialogButtonBox, QListWidget, QListWidgetItem,
-                             QFrame, QMenu, QAction, QShortcut)
-from PyQt5.QtGui import QIcon
 
+# third party imports
+from qtpy.QtCore import Qt, QSize
+from qtpy.QtGui import QIcon
 if TYPE_CHECKING:
-    from gui_session import SignaledSession
+    # FIX : QAction not found by pylance
+    from qtpy.QtGui import QAction, QShortcut
+from qtpy.QtWidgets import (QDialogButtonBox, QListWidget, QListWidgetItem,
+                             QFrame, QMenu, QAction, QShortcut) # type:ignore
 
+# Imports from src/shared
 import ray
 import client_properties_dialog
+import osc_paths.ray as r
 
+# Local imports
 from gui_tools import RS, _translate, is_dark_theme, get_app_icon
 from child_dialogs import ChildDialog
 
+# Import UIs made with Qt-Designer
 import ui.template_slot
 import ui.remove_template
 import ui.add_application
 
+if TYPE_CHECKING:
+    from gui_session import SignaledSession
 
-TEMPLATE_NAME_DATA = Qt.UserRole
-DISPLAY_NAME_DATA = Qt.UserRole + 1
+TEMPLATE_NAME_DATA = Qt.ItemDataRole.UserRole
+DISPLAY_NAME_DATA = Qt.ItemDataRole.UserRole + 1
 
 
 class TemplateSlot(QFrame):
@@ -49,7 +59,7 @@ class TemplateSlot(QFrame):
                                       self._user_menu)
         act_remove_template.triggered.connect(self.remove_template)
         self._user_menu.addAction(act_remove_template)
-        self.ui.toolButtonUser.setMenu(self._user_menu)
+        self.ui.toolButtonUser.setMenu(self._user_menu) # type:ignore
 
         self.ui.toolButtonFavorite.set_session(self.session)
         self.ui.toolButtonFavorite.set_template(
@@ -58,7 +68,7 @@ class TemplateSlot(QFrame):
 
         if is_dark_theme(self):
             self.ui.toolButtonUser.setIcon(
-                QIcon(':scalable/breeze-dark/im-user.svg'))
+                QIcon(':scalable/breeze-dark/im-user.svg')) # type:ignore
             self.ui.toolButtonFavorite.set_dark_theme()
 
     def update_client_data(self, *args):
@@ -86,23 +96,22 @@ class TemplateSlot(QFrame):
         return self.ui.label.text()
 
     def remove_template(self):
-        add_app_dialog = self._list_widget.parent()
-        
-        # 'if' statement only for typing, should always be True.
-        if isinstance(add_app_dialog, AddApplicationDialog):
-            add_app_dialog.remove_template(self._name, self._factory)
+        add_app_dialog: AddApplicationDialog = \
+            self._list_widget.parent() # type:ignore
+        add_app_dialog.remove_template(self._name, self._factory)
 
     def set_as_favorite(self, yesno: bool):
         self.ui.toolButtonFavorite.set_as_favorite(yesno)
 
     def mouseDoubleClickEvent(self, event):
-        self._list_widget.parent().accept()
+        self._list_widget.parent().accept() # type:ignore
 
 
 class TemplateItem(QListWidgetItem):
     def __init__(self, parent: QListWidget, session: 'SignaledSession',
                  name: str, factory: bool):
-        QListWidgetItem.__init__(self, parent, QListWidgetItem.UserType + 1)
+        QListWidgetItem.__init__(
+            self, parent, QListWidgetItem.ItemType.UserType + 1)
 
         self.client_data = ray.ClientData()
         self._widget = TemplateSlot(parent, session,
@@ -164,7 +173,8 @@ class RemoveTemplateDialog(ChildDialog):
         self.ui.label.setText(
             _translate(
                 'add_app_dialog',
-                '<p>Are you sure to want to remove<br>the template "%s" and all its files ?</p>')
+                '<p>Are you sure to want to remove<br>'
+                'the template "%s" and all its files ?</p>')
                 % template_name)
 
         self.ui.pushButtonCancel.setFocus()
@@ -186,21 +196,26 @@ class AddApplicationDialog(ChildDialog):
             'AddApplication/ray_hack_box', True, type=bool))
         self.ui.widgetTemplateInfos.setVisible(False)
 
-        self.ui.checkBoxFactory.stateChanged.connect(self._factory_box_changed)
-        self.ui.checkBoxUser.stateChanged.connect(self._user_box_changed)
-        self.ui.checkBoxNsm.stateChanged.connect(self._nsm_box_changed)
-        self.ui.checkBoxRayHack.stateChanged.connect(self._ray_hack_box_changed)
-        self.ui.pushButtonRefresh.clicked.connect(self._refresh_database)
+        self.ui.checkBoxFactory.stateChanged.connect(
+            self._factory_box_changed)
+        self.ui.checkBoxUser.stateChanged.connect(
+            self._user_box_changed)
+        self.ui.checkBoxNsm.stateChanged.connect(
+            self._nsm_box_changed)
+        self.ui.checkBoxRayHack.stateChanged.connect(
+            self._ray_hack_box_changed)
+        self.ui.pushButtonRefresh.clicked.connect(
+            self._refresh_database)
         self._refresh_shortcut = QShortcut('F5', self)
         self._refresh_shortcut.activated.connect(self._refresh_database)
 
         self.ui.templateList.currentItemChanged.connect(
             self._current_item_changed)
-        self.ui.templateList.setFocus(Qt.OtherFocusReason)
+        self.ui.templateList.setFocus(Qt.FocusReason.OtherFocusReason) # type:ignore
         self.ui.filterBar.textEdited.connect(self._update_filtered_list)
         self.ui.filterBar.up_down_pressed.connect(self._up_down_pressed)
 
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
 
         self.user_menu = QMenu()
         act_remove_template = QAction(QIcon.fromTheme('edit-delete-remove'),
@@ -208,7 +223,7 @@ class AddApplicationDialog(ChildDialog):
                                       self.user_menu)
         act_remove_template.triggered.connect(self._remove_current_template)
         self.user_menu.addAction(act_remove_template)
-        self.ui.toolButtonUser.setMenu(self.user_menu)
+        self.ui.toolButtonUser.setMenu(self.user_menu) # type:ignore
         self.ui.toolButtonFavorite.set_session(self.session)
         self.ui.widgetNonSaveable.setVisible(False)
         self.ui.toolButtonAdvanced.clicked.connect(
@@ -217,7 +232,7 @@ class AddApplicationDialog(ChildDialog):
 
         if is_dark_theme(self):
             self.ui.toolButtonUser.setIcon(
-                QIcon(':scalable/breeze-dark/im-user.svg'))
+                QIcon(':scalable/breeze-dark/im-user.svg')) # type:ignore
             self.ui.toolButtonFavorite.set_dark_theme()
             self.ui.toolButtonNoSave.setIcon(
                 QIcon(':scalable/breeze-dark/document-nosave.svg'))
@@ -235,12 +250,12 @@ class AddApplicationDialog(ChildDialog):
         self.signaler.favorite_added.connect(self._favorite_added)
         self.signaler.favorite_removed.connect(self._favorite_removed)
 
-        self.to_daemon('/ray/server/list_user_client_templates')
-        self.to_daemon('/ray/server/list_factory_client_templates')
+        self.to_daemon(r.server.LIST_USER_CLIENT_TEMPLATES)
+        self.to_daemon(r.server.LIST_FACTORY_CLIENT_TEMPLATES)
         self.listing_finished = 0
 
-        self.user_template_list = []
-        self.factory_template_list = []
+        self.user_template_list = list[str]()
+        self.factory_template_list = list[str]()
 
         self._server_will_accept = False
         self.has_selection = False
@@ -252,7 +267,7 @@ class AddApplicationDialog(ChildDialog):
     def _favorite_added(self, template_name: str, template_icon: str,
                         factory: bool, display_name: str):
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
             if item is None:
                 continue
 
@@ -265,7 +280,7 @@ class AddApplicationDialog(ChildDialog):
 
     def _favorite_removed(self, template_name: str, factory: bool):
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
             if item is None:
                 continue
 
@@ -305,11 +320,11 @@ class AddApplicationDialog(ChildDialog):
         self.ui.templateList.clear()
         self.user_template_list.clear()
         self.factory_template_list.clear()
-        self.to_daemon('/ray/server/clear_client_templates_database')
-        self.to_daemon('/ray/server/list_user_client_templates')
-        self.to_daemon('/ray/server/list_factory_client_templates')
+        self.to_daemon(r.server.CLEAR_CLIENT_TEMPLATES_DATABASE)
+        self.to_daemon(r.server.LIST_USER_CLIENT_TEMPLATES)
+        self.to_daemon(r.server.LIST_FACTORY_CLIENT_TEMPLATES)
 
-    def _add_user_templates(self, template_list):
+    def _add_user_templates(self, template_list: list[str]):
         for template_name in template_list:
             if template_name in self.user_template_list:
                 continue
@@ -322,13 +337,13 @@ class AddApplicationDialog(ChildDialog):
             if self.session.is_favorite(template_name, False):
                 item.set_as_favorite(True)
 
-            self.ui.templateList.addItem(item)
+            self.ui.templateList.addItem(item) # type:ignore
 
         self.ui.templateList.sortItems()
 
         self._update_filtered_list()
 
-    def _add_factory_templates(self, template_list):
+    def _add_factory_templates(self, template_list: list[str]):
         for template_name in template_list:
             if template_name in self.factory_template_list:
                 continue
@@ -341,7 +356,7 @@ class AddApplicationDialog(ChildDialog):
             if self.session.is_favorite(template_name, True):
                 item.set_as_favorite(True)
 
-            self.ui.templateList.addItem(item)
+            self.ui.templateList.addItem(item) # type:ignore
 
         self.ui.templateList.sortItems()
 
@@ -356,7 +371,7 @@ class AddApplicationDialog(ChildDialog):
         display_name = args[2]
 
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
 
             if item.matches_with(factory, template_name):
                 if display_name:
@@ -371,7 +386,7 @@ class AddApplicationDialog(ChildDialog):
         template_name = args[1]
 
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
 
             if item.matches_with(factory, template_name):
                 item.update_ray_hack_data(*args[2:])
@@ -384,7 +399,7 @@ class AddApplicationDialog(ChildDialog):
         template_name = args[1]
 
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
 
             if item.matches_with(factory, template_name):
                 item.update_ray_net_data(*args[2:])
@@ -401,7 +416,7 @@ class AddApplicationDialog(ChildDialog):
 
         # hide all non matching items
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
             template_name: str = item.data(TEMPLATE_NAME_DATA)
 
             if not filter_text.lower() in template_name.lower():
@@ -414,11 +429,11 @@ class AddApplicationDialog(ChildDialog):
                 item.setHidden(True)
 
             if item.client_data is not None:
-                if (item.client_data.protocol == ray.Protocol.RAY_HACK
+                if (item.client_data.protocol is ray.Protocol.RAY_HACK
                         and not self.ui.checkBoxRayHack.isChecked()):
                     item.setHidden(True)
 
-                if (item.client_data.protocol != ray.Protocol.RAY_HACK
+                if (item.client_data.protocol is not ray.Protocol.RAY_HACK
                         and not self.ui.checkBoxNsm.isChecked()):
                     item.setHidden(True)
 
@@ -435,7 +450,7 @@ class AddApplicationDialog(ChildDialog):
             if self.ui.filterBar.text():
                 self.ui.filterBar.setStyleSheet(
                     "QLineEdit { background-color: red}")
-            self.ui.templateList.setCurrentItem(None)
+            self.ui.templateList.setCurrentItem(None) # type:ignore
         else:
             self.ui.filterBar.setStyleSheet("")
             self.ui.templateList.scrollTo(self.ui.templateList.currentIndex())
@@ -443,7 +458,7 @@ class AddApplicationDialog(ChildDialog):
     def _up_down_pressed(self, key):
         row = self.ui.templateList.currentRow()
 
-        if key == Qt.Key_Up:
+        if key == Qt.Key.Key_Up:
             if row == 0:
                 return
             row -= 1
@@ -452,7 +467,7 @@ class AddApplicationDialog(ChildDialog):
                     return
                 row -= 1
 
-        elif key == Qt.Key_Down:
+        elif key == Qt.Key.Key_Down:
             if row == self.ui.templateList.count() - 1:
                 return
             row += 1
@@ -463,7 +478,7 @@ class AddApplicationDialog(ChildDialog):
 
         self.ui.templateList.setCurrentRow(row)
 
-    def _update_template_infos(self, item: TemplateItem):
+    def _update_template_infos(self, item: TemplateItem | None):
         self.ui.widgetTemplateInfos.setVisible(bool(item))
         self.ui.widgetNoTemplate.setVisible(not bool(item))
 
@@ -479,7 +494,7 @@ class AddApplicationDialog(ChildDialog):
         self.ui.labelTemplateName.setText(item.get_display_name())
         #self.ui.labelTemplateName.setText(template_name)
         self.ui.labelDescription.setText(cdata.description)
-        self.ui.labelProtocol.setText(ray.protocol_to_str(cdata.protocol))
+        self.ui.labelProtocol.setText(cdata.protocol.to_string())
         self.ui.labelExecutable.setText(cdata.executable_path)
         self.ui.labelLabel.setText(cdata.label)
         self.ui.labelName.setText(cdata.name)
@@ -487,7 +502,7 @@ class AddApplicationDialog(ChildDialog):
         for widget in (self.ui.labelProtocolTitle,
                        self.ui.labelProtocolColon,
                        self.ui.labelProtocol):
-            widget.setVisible(bool(cdata.protocol != ray.Protocol.NSM))
+            widget.setVisible(bool(cdata.protocol is not ray.Protocol.NSM))
 
         for widget in (self.ui.labelLabelTitle,
                        self.ui.labelLabelColon,
@@ -497,7 +512,8 @@ class AddApplicationDialog(ChildDialog):
         for widget in (self.ui.labelNameTitle,
                        self.ui.labelNameColon,
                        self.ui.labelName):
-            widget.setVisible(bool(cdata.protocol == ray.Protocol.NSM))
+            widget.setVisible(bool(
+                cdata.protocol in (ray.Protocol.NSM, ray.Protocol.INTERNAL)))
 
         self.ui.toolButtonUser.setVisible(not item.is_factory)
         self.ui.toolButtonFavorite.set_template(
@@ -508,14 +524,14 @@ class AddApplicationDialog(ChildDialog):
 
         self.ui.widgetNonSaveable.setVisible(bool(
             cdata.ray_hack is not None
-            and cdata.protocol == ray.Protocol.RAY_HACK
+            and cdata.protocol is ray.Protocol.RAY_HACK
             and cdata.ray_hack.no_save_level > 0))
 
         # little security
         # client_properties_dialog could crash if ray_hack has not been updated yet
         # (never seen this appears, but it could with slow systems)
         self.ui.toolButtonAdvanced.setEnabled(
-            bool(cdata.protocol != ray.Protocol.RAY_HACK
+            bool(cdata.protocol is not ray.Protocol.RAY_HACK
                  or cdata.ray_hack is not None))
 
     def _current_item_changed(self, item, previous_item):
@@ -524,7 +540,7 @@ class AddApplicationDialog(ChildDialog):
         self._prevent_ok()
 
     def _tool_button_advanced_clicked(self):
-        item: TemplateItem = self.ui.templateList.currentItem()
+        item: TemplateItem = self.ui.templateList.currentItem() # type:ignore
         if item is None:
             return
 
@@ -538,7 +554,7 @@ class AddApplicationDialog(ChildDialog):
         self.ui.lineEditUniqueId.setText(text.replace(' ', '_'))
 
     def _prevent_ok(self):
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled( # type:ignore
             bool(self._server_will_accept and self.has_selection))
 
     def _remove_current_template(self):
@@ -548,7 +564,7 @@ class AddApplicationDialog(ChildDialog):
 
         self.remove_template(item.data(TEMPLATE_NAME_DATA), False)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         self._server_will_accept = bool(
             server_status not in (
                 ray.ServerStatus.OFF,
@@ -556,10 +572,11 @@ class AddApplicationDialog(ChildDialog):
             and not self.server_copying)
         self._prevent_ok()
 
-    def get_selected_template(self) -> tuple:
-        item: TemplateItem = self.ui.templateList.currentItem()
+    def get_selected_template(self) -> tuple[str, bool]:
+        item: TemplateItem = self.ui.templateList.currentItem() # type:ignore
         if item:
             return (item.data(TEMPLATE_NAME_DATA), item.is_factory)
+        return ('', False)
 
     def get_selected_unique_id(self) -> str:
         return self.ui.lineEditUniqueId.text()
@@ -570,17 +587,17 @@ class AddApplicationDialog(ChildDialog):
         if not dialog.result():
             return
 
-        self.to_daemon('/ray/server/remove_client_template', template_name)
+        self.to_daemon(r.server.REMOVE_CLIENT_TEMPLATE, template_name)
 
         for i in range(self.ui.templateList.count()):
-            item: TemplateItem = self.ui.templateList.item(i)
+            item: TemplateItem = self.ui.templateList.item(i) # type:ignore
 
             if (not item.is_factory
                     and template_name == item.data(TEMPLATE_NAME_DATA)):
                 item.setHidden(True)
                 if item == self.ui.templateList.currentItem():
                     self._update_template_infos(None)
-                self.ui.templateList.removeItemWidget(item)
+                self.ui.templateList.removeItemWidget(item) # type:ignore
                 break
 
     def save_check_boxes(self):
