@@ -1,24 +1,33 @@
+
+# Imports from standard library
 import os
-import sys
+import logging
 import time
 import subprocess
 from typing import TYPE_CHECKING
 
-from PyQt5.QtWidgets import (
+# third party imports
+from qtpy.QtWidgets import (
     QDialog, QDialogButtonBox, QCompleter, QMessageBox,
     QFileDialog, QApplication, QListWidgetItem)
-from PyQt5.QtGui import QIcon, QPixmap, QGuiApplication, QKeyEvent
-from PyQt5.QtCore import Qt, QTimer
+from qtpy.QtGui import (
+    QIcon, QPixmap, QGuiApplication, QKeyEvent, QDesktopServices)
+from qtpy.QtCore import Qt, QTimer, QUrl
 
+# Imports from src/shared
+from osclib import Address, verified_address
 import ray
+import osc_paths
+import osc_paths.ray as r
+import osc_paths.ray.gui as rg
+
+# Local imports
 import client_properties_dialog
 from gui_server_thread import GuiServerThread
 from gui_tools import (ErrDaemon, _translate, get_app_icon,
                        CommandLineArgs, RS, is_dark_theme)
-if TYPE_CHECKING:
-    from main_window import MainWindow
-    from gui_client import Client
 
+# Import UIs made with Qt-Designer
 import ui.new_session
 import ui.save_template_session
 import ui.client_trash
@@ -44,6 +53,13 @@ import ui.systray_close
 import ui.startup_dialog
 import ui.error_dialog
 
+if TYPE_CHECKING:
+    from main_window import MainWindow
+    from gui_client import Client
+
+
+_logger = logging.getLogger(__name__)
+
 
 class ChildDialog(QDialog):
     def __init__(self, parent: 'MainWindow'):
@@ -57,23 +73,22 @@ class ChildDialog(QDialog):
 
         self._root_folder_file_dialog = None
         self._root_folder_message_box = QMessageBox(
-            QMessageBox.Critical,
+            QMessageBox.Icon.Critical,
             _translate('root_folder_dialogs', 'unwritable dir'),
-            '', QMessageBox.NoButton, self)
+            '', QMessageBox.StandardButton.NoButton, self)
 
         self.server_copying = parent.server_copying
 
     @classmethod
     def to_daemon(cls, *args):
         server = GuiServerThread.instance()
-        if server:
+        if server is not None:
             server.to_daemon(*args)
         else:
-            sys.stderr.write('Error No GUI OSC Server, can not send %s.\n'
-                             % args)
+            _logger.error(f'No GUI OSC Server, can not send {args}.')
 
-    def _server_status_changed(self, server_status: int):
-        pass
+    def _server_status_changed(self, server_status: ray.ServerStatus):
+        ...
 
     def _server_copying(self, copying: bool):
         self.server_copying = copying
@@ -87,8 +102,8 @@ class ChildDialog(QDialog):
                 _translate("root_folder_dialogs",
                         "Choose root folder for sessions"),
                 CommandLineArgs.session_root)
-            self._root_folder_file_dialog.setFileMode(QFileDialog.Directory)
-            self._root_folder_file_dialog.setOption(QFileDialog.ShowDirsOnly)
+            self._root_folder_file_dialog.setFileMode(QFileDialog.FileMode.Directory)
+            self._root_folder_file_dialog.setOption(QFileDialog.Option.ShowDirsOnly)
         else:
             self._root_folder_file_dialog.setDirectory(CommandLineArgs.session_root)
 
@@ -124,10 +139,10 @@ class ChildDialog(QDialog):
             return
 
         RS.settings.setValue('default_session_root', root_folder)
-        self.to_daemon('/ray/server/change_root', root_folder)
+        self.to_daemon(r.server.CHANGE_ROOT, root_folder)
 
     def parent(self) -> 'MainWindow':
-        super().parent()
+        return super().parent() # type:ignore
 
     def leaveEvent(self, event):
         parent = self.parent()
@@ -153,8 +168,8 @@ class NewSessionDialog(ChildDialog):
 
         self.ui.currentSessionsFolder.setText(CommandLineArgs.session_root)
         self.ui.toolButtonFolder.clicked.connect(self._change_root_folder)
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
-        self.ui.lineEdit.setFocus(Qt.OtherFocusReason)
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
+        self.ui.lineEdit.setFocus(Qt.FocusReason.OtherFocusReason) # type:ignore
         self.ui.lineEdit.textChanged.connect(self._text_changed)
 
         self.session_list = []
@@ -166,7 +181,7 @@ class NewSessionDialog(ChildDialog):
         self.signaler.session_template_found.connect(self._add_templates_to_list)
         self.signaler.root_changed.connect(self._root_changed)
 
-        self.to_daemon('/ray/server/list_sessions', 1)
+        self.to_daemon(r.server.LIST_SESSIONS, 1)
 
         if self._is_duplicate:
             self.ui.labelTemplate.setVisible(False)
@@ -177,7 +192,7 @@ class NewSessionDialog(ChildDialog):
             self.setWindowTitle(_translate('Duplicate', 'Duplicate Session'))
         else:
             self.ui.frameOriginalSession.setVisible(False)
-            self.to_daemon('/ray/server/list_session_templates')
+            self.to_daemon(r.server.LIST_SESSION_TEMPLATES)
 
         if not self.daemon_manager.is_local:
             self.ui.toolButtonFolder.setVisible(False)
@@ -191,15 +206,15 @@ class NewSessionDialog(ChildDialog):
         self._text_is_valid = False
 
         self._completer = QCompleter(self.sub_folders)
-        self.ui.lineEdit.setCompleter(self._completer)
+        self.ui.lineEdit.setCompleter(self._completer) # type:ignore
 
         self._server_status_changed(self.session.server_status)
         
         self._text_was_empty = True
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         self.ui.toolButtonFolder.setEnabled(
-            bool(server_status == ray.ServerStatus.OFF))
+            bool(server_status is ray.ServerStatus.OFF))
 
         self._server_will_accept = bool(
             server_status in (
@@ -207,9 +222,9 @@ class NewSessionDialog(ChildDialog):
                 ray.ServerStatus.READY) and not self.server_copying)
         if self._is_duplicate:
             self._server_will_accept = bool(
-                server_status == ray.ServerStatus.READY and not self.server_copying)
+                server_status is ray.ServerStatus.READY and not self.server_copying)
 
-        if server_status != ray.ServerStatus.OFF:
+        if server_status is not ray.ServerStatus.OFF:
             if self._root_folder_file_dialog is not None:
                 self._root_folder_file_dialog.reject()
             self._root_folder_message_box.reject()
@@ -220,7 +235,7 @@ class NewSessionDialog(ChildDialog):
         self.ui.currentSessionsFolder.setText(session_root)
         self.session_list.clear()
         self.sub_folders.clear()
-        self.to_daemon('/ray/server/list_sessions', 1)
+        self.to_daemon(r.server.LIST_SESSIONS, 1)
 
     def _init_templates_combo_box(self):
         self.ui.comboBoxTemplate.clear()
@@ -287,7 +302,7 @@ class NewSessionDialog(ChildDialog):
         self.sub_folders.sort()
         del self._completer
         self._completer = QCompleter([f + '/' for f in self.sub_folders])
-        self.ui.lineEdit.setCompleter(self._completer)
+        self.ui.lineEdit.setCompleter(self._completer) # type:ignore
 
         if not session_names:
             # all sessions are listed, pre-fill last subfolder
@@ -318,7 +333,8 @@ class NewSessionDialog(ChildDialog):
 
         if self._text_was_empty:
             if text:
-                self._completer.setCompletionMode(QCompleter.PopupCompletion)
+                self._completer.setCompletionMode(
+                    QCompleter.CompletionMode.PopupCompletion)
                 self._completer.complete()
                 self._text_was_empty = False
         
@@ -329,8 +345,9 @@ class NewSessionDialog(ChildDialog):
     def _set_completer_for_empty_text(self):
         del self._completer
         self._completer = QCompleter([f + '/' for f in self.sub_folders])
-        self._completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
-        self.ui.lineEdit.setCompleter(self._completer)
+        self._completer.setCompletionMode(
+            QCompleter.CompletionMode.UnfilteredPopupCompletion)
+        self.ui.lineEdit.setCompleter(self._completer) # type:ignore
         QTimer.singleShot(50, self._completer.complete)
 
     def _show_completer_at_start(self):
@@ -341,7 +358,7 @@ class NewSessionDialog(ChildDialog):
         QTimer.singleShot(800, self._show_completer_at_start)
 
     def _prevent_ok(self):
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled( # type:ignore
             bool(self._server_will_accept and self._text_is_valid))
 
     def get_session_short_path(self)->str:
@@ -370,12 +387,12 @@ class AbstractSaveTemplateDialog(ChildDialog):
             "session template", "Update the template")
         self._create_template_text = self.ui.pushButtonAccept.text()
         self._overwrite_message_box = QMessageBox(
-            QMessageBox.Question,
+            QMessageBox.Icon.Question,
             _translate(
                     'session template',
                     'Overwrite Template ?'),
             '',
-            QMessageBox.Yes | QMessageBox.No,
+            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
             self)
 
         self.template_list = []
@@ -409,7 +426,7 @@ class AbstractSaveTemplateDialog(ChildDialog):
             self._overwrite_message_box.exec()
 
             if (self._overwrite_message_box.clickedButton()
-                    == self._overwrite_message_box.button(QMessageBox.No)):
+                    == self._overwrite_message_box.button(QMessageBox.StandardButton.No)):
                 return
         self.accept()
 
@@ -432,14 +449,14 @@ class SaveTemplateSessionDialog(AbstractSaveTemplateDialog):
         self.ui.labelLabel.setText(self.session.get_short_path())
 
         self.signaler.session_template_found.connect(self._add_templates_to_list)
-        self.to_daemon('/ray/server/list_session_templates')
+        self.to_daemon(r.server.LIST_SESSION_TEMPLATES)
 
         self._server_status_changed(self.session.server_status)
 
-    def _server_status_changed(self, server_status):
-        self._server_will_accept = bool(server_status == ray.ServerStatus.READY)
+    def _server_status_changed(self, server_status: ray.ServerStatus):
+        self._server_will_accept = bool(server_status is ray.ServerStatus.READY)
 
-        if server_status == ray.ServerStatus.OFF:
+        if server_status is ray.ServerStatus.OFF:
             self._overwrite_message_box.reject()
             self.reject()
 
@@ -464,13 +481,13 @@ class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
         self.signaler.user_client_template_found.connect(
             self._add_templates_to_list)
 
-        self.to_daemon('/ray/server/list_user_client_templates')
+        self.to_daemon(r.server.LIST_USER_CLIENT_TEMPLATES)
         self.ui.lineEdit.setText(client.template_origin)
         self.ui.lineEdit.selectAll()
         self.ui.lineEdit.setFocus()
         self._server_status_changed(self.session.server_status)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         self._server_will_accept = bool(
             server_status not in (
                 ray.ServerStatus.OFF,
@@ -502,16 +519,17 @@ class ClientTrashDialog(ChildDialog):
         self.ui.pushButtonCancel.setFocus()
 
         self._remove_client_message_box = QMessageBox(
-            QMessageBox.Warning,
+            QMessageBox.Icon.Warning,
             _translate('trashed_client', 'Remove definitely'),
             _translate('trashed_client',
                 "Are you sure to want to remove definitely this client and all its files ?"),
-            QMessageBox.Ok | QMessageBox.Cancel,
+            QMessageBox.StandardButton.Ok | QMessageBox.StandardButton.Cancel,
             self
             )
-        self._remove_client_message_box.setDefaultButton(QMessageBox.Cancel)
+        self._remove_client_message_box.setDefaultButton(
+            QMessageBox.StandardButton.Cancel)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         if server_status in (ray.ServerStatus.CLOSE,
                              ray.ServerStatus.OFF,
                              ray.ServerStatus.OUT_SAVE,
@@ -524,11 +542,12 @@ class ClientTrashDialog(ChildDialog):
         self._remove_client_message_box.exec()
 
         if (self._remove_client_message_box.clickedButton()
-                != self._remove_client_message_box.button(QMessageBox.Ok)):
+                != self._remove_client_message_box.button(
+                    QMessageBox.StandardButton.Ok)):
             return
 
         self.to_daemon(
-            '/ray/trashed_client/remove_definitely',
+            r.trashed_client.REMOVE_DEFINITELY,
             self.client_data.client_id)
         self.reject()
 
@@ -548,11 +567,11 @@ class AbortSessionDialog(ChildDialog):
 
         self.ui.pushButtonAbort.clicked.connect(self.accept)
         self.ui.pushButtonCancel.clicked.connect(self.reject)
-        self.ui.pushButtonCancel.setFocus(Qt.OtherFocusReason)
+        self.ui.pushButtonCancel.setFocus(Qt.FocusReason.OtherFocusReason) # type:ignore
 
         self._server_status_changed(self.session.server_status)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         self.ui.pushButtonAbort.setEnabled(
             not bool(
                 server_status in (
@@ -573,14 +592,14 @@ class AbortServerCopyDialog(ChildDialog):
 
         self._server_status_changed(self.session.server_status)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         if server_status not in (
                 ray.ServerStatus.PRECOPY,
                 ray.ServerStatus.COPY):
             self.reject()
 
     def _set_progress(self, progress: float):
-        self.ui.progressBar.setValue(progress * 100)
+        self.ui.progressBar.setValue(int(progress * 100))
 
 
 class AbortClientCopyDialog(ChildDialog):
@@ -597,9 +616,9 @@ class AbortClientCopyDialog(ChildDialog):
         if client_id != self._client_id:
             return
 
-        self.ui.progressBar.setValue(progress * 100)
+        self.ui.progressBar.setValue(int(progress * 100))
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         if not self.server_copying:
             self.reject()
 
@@ -631,8 +650,8 @@ class SessionNotesDialog(ChildDialog):
         self._anti_timer = False
         self.notes_updated()
 
-    def _server_status_changed(self, server_status):
-        if server_status == ray.ServerStatus.OFF:
+    def _server_status_changed(self, server_status: ray.ServerStatus):
+        if server_status is ray.ServerStatus.OFF:
             self.server_off = True
             if self._message_box is not None:
                 self._message_box.close()
@@ -649,19 +668,19 @@ class SessionNotesDialog(ChildDialog):
         notes = self.ui.plainTextEdit.toPlainText()
         if len(notes) >= 65000:
             self._message_box = QMessageBox(
-                QMessageBox.Critical,
+                QMessageBox.Icon.Critical,
                 _translate('session_notes', 'Too long notes'),
                 _translate('session_notes',
                            "<p>Because notes are spread to the OSC server,<br>"
                            "they can't be longer than 65000 characters.<br>Sorry !</p>"),
-                QMessageBox.Cancel,
+                QMessageBox.StandardButton.Cancel,
                 self)
             self._message_box.exec()
             self.ui.plainTextEdit.setPlainText(notes[:64999])
             return
 
         self.session.notes = notes
-        self.to_daemon('/ray/session/set_notes', self.session.notes)
+        self.to_daemon(r.session.SET_NOTES, self.session.notes)
 
     def update_session(self):
         self.setWindowTitle(_translate('notes_dialog', "%s Notes - %s")
@@ -676,7 +695,7 @@ class SessionNotesDialog(ChildDialog):
         RS.settings.setValue('SessionNotes/geometry', self.saveGeometry())
         RS.settings.setValue('SessionNotes/position', self.pos())
         if not self.server_off:
-            self.to_daemon('/ray/session/hide_notes')
+            self.to_daemon(r.session.HIDE_NOTES)
         ChildDialog.closeEvent(self, event)
 
 
@@ -696,7 +715,7 @@ class QuitAppDialog(ChildDialog):
         ChildDialog.__init__(self, parent)
         self.ui = ui.quit_app.Ui_DialogQuitApp()
         self.ui.setupUi(self)
-        self.ui.pushButtonCancel.setFocus(Qt.OtherFocusReason)
+        self.ui.pushButtonCancel.setFocus(Qt.FocusReason.OtherFocusReason) # type:ignore
         self.ui.pushButtonSaveQuit.clicked.connect(self._close_session)
         self.ui.pushButtonQuitNoSave.clicked.connect(self._abort_session)
         self.ui.pushButtonDaemon.clicked.connect(self._leave_daemon_running)
@@ -711,21 +730,21 @@ class QuitAppDialog(ChildDialog):
             self.ui.pushButtonDaemon.setVisible(False)
         self._server_status_changed(self.session.server_status)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         if server_status == ray.ServerStatus.OFF:
             self.accept()
             return
 
         self.ui.pushButtonSaveQuit.setEnabled(
-            bool(server_status == ray.ServerStatus.READY))
+            bool(server_status is ray.ServerStatus.READY))
         self.ui.pushButtonQuitNoSave.setEnabled(
-            bool(server_status != ray.ServerStatus.CLOSE))
+            bool(server_status is not ray.ServerStatus.CLOSE))
 
     def _close_session(self):
-        self.to_daemon('/ray/session/close')
+        self.to_daemon(r.session.CLOSE)
 
     def _abort_session(self):
-        self.to_daemon('/ray/session/abort')
+        self.to_daemon(r.session.ABORT)
 
     def _leave_daemon_running(self):
         if CommandLineArgs.under_nsm:
@@ -745,11 +764,6 @@ class WrongVersionLocalDialog(ChildDialog):
         self.ui.pushButtonQuitNoSave.clicked.connect(self._abort_session)
         self.ui.pushButtonDaemon.clicked.connect(self._leave_daemon_running)
 
-        #original_text = self.ui.labelMainText.text()
-        #self.ui.labelMainText.setText(
-            #original_text %
-            #('<strong>%s</strong>' %
-             #self.session.name))
         self.ui.labelMainText.setText(
             _translate(
                 'wrong_version',
@@ -794,9 +808,9 @@ class NewExecutableDialog(ChildDialog):
             self.ui.comboBoxPrefixMode.toolTip())
         self.ui.labelClientId.setToolTip(self.ui.lineEditClientId.toolTip())
 
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
 
-        self.ui.lineEdit.setFocus(Qt.OtherFocusReason)
+        self.ui.lineEdit.setFocus(Qt.FocusReason.OtherFocusReason) # type:ignore
         self.ui.lineEdit.textChanged.connect(self._check_allow)
         self.ui.checkBoxNsm.stateChanged.connect(self._check_allow)
 
@@ -809,24 +823,24 @@ class NewExecutableDialog(ChildDialog):
             _translate('new_executable', 'Client Name'))
         self.ui.comboBoxPrefixMode.addItem(
             _translate('new_executable', 'Session Name'))
-        self.ui.comboBoxPrefixMode.setCurrentIndex(2)
+        self.ui.comboBoxPrefixMode.setCurrentIndex(1)
 
         self.ui.comboBoxPrefixMode.currentIndexChanged.connect(
             self._prefix_mode_changed)
 
         self.signaler.new_executable.connect(self._add_executable_to_completer)
-        self.to_daemon('/ray/server/list_path')
+        self.to_daemon(r.server.LIST_PATH)
 
         self.exec_list = []
 
         self._completer = QCompleter(self.exec_list)
-        self.ui.lineEdit.setCompleter(self._completer)
+        self.ui.lineEdit.setCompleter(self._completer) # type:ignore
 
         self.ui.lineEdit.returnPressed.connect(self._close_now)
 
         self._server_status_changed(self.session.server_status)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         if server_status in (ray.ServerStatus.OUT_SAVE,
                              ray.ServerStatus.OUT_SNAPSHOT,
                              ray.ServerStatus.WAIT_USER,
@@ -847,7 +861,7 @@ class NewExecutableDialog(ChildDialog):
 
         del self._completer
         self._completer = QCompleter(self.exec_list)
-        self.ui.lineEdit.setCompleter(self._completer)
+        self.ui.lineEdit.setCompleter(self._completer) # type:ignore
 
     def _is_allowed(self) -> bool:
         nsm = self.ui.checkBoxNsm.isChecked()
@@ -856,13 +870,13 @@ class NewExecutableDialog(ChildDialog):
 
     def _check_allow(self):
         allow = self._is_allowed()
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(allow)
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(allow) # type:ignore
 
     def _close_now(self):
         if self._is_allowed():
             self.accept()
 
-    def get_selection(self)->tuple:
+    def get_selection(self) -> tuple[str, bool, bool, int, str, str, bool]:
         return (self.ui.lineEdit.text(),
                 self.ui.checkBoxStartClient.isChecked(),
                 not self.ui.checkBoxNsm.isChecked(),
@@ -902,9 +916,11 @@ class StopClientDialog(ChildDialog):
 
     def _save_and_stop(self):
         self._wait_for_save = True
-        self.to_daemon('/ray/client/save', self._client_id)
+        self.to_daemon(r.client.SAVE, self._client_id)
 
     def _check_box_clicked(self, state):
+        if self.client is None:
+            return
         self.client.check_last_save = not bool(state)
         self.client.send_properties_to_daemon()
 
@@ -913,7 +929,7 @@ class StopClientDialog(ChildDialog):
             self.reject()
             return
 
-        if status == ray.ClientStatus.READY and self._wait_for_save:
+        if status is ray.ClientStatus.READY and self._wait_for_save:
             self._wait_for_save = False
             self.accept()
 
@@ -932,7 +948,7 @@ class StopClientNoSaveDialog(ChildDialog):
             self.client.status_changed.connect(self._server_updates_client_status)
 
         self.ui.checkBox.stateChanged.connect(self._check_box_clicked)
-        self.ui.pushButtonCancel.setFocus(True)
+        self.ui.pushButtonCancel.setFocus()
 
     def _server_updates_client_status(self, status: int):
         if status in (ray.ClientStatus.STOPPED, ray.ClientStatus.REMOVED):
@@ -940,6 +956,8 @@ class StopClientNoSaveDialog(ChildDialog):
             return
 
     def _check_box_clicked(self, state):
+        if self.client is None:
+            return
         self.client.check_last_save = not bool(state)
         self.client.send_properties_to_daemon()
 
@@ -960,20 +978,22 @@ class ClientRenameDialog(ChildDialog):
         self.ui.checkBoxIdRename.stateChanged.connect(
             self._id_rename_state_changed)
 
-        if client.protocol not in (ray.Protocol.NSM, ray.Protocol.RAY_HACK):
+        if client.protocol not in (ray.Protocol.NSM,
+                                   ray.Protocol.RAY_HACK,
+                                   ray.Protocol.INTERNAL):
             self.ui.checkBoxIdRename.setVisible(False)
 
         self._change_box_text_with_status(client.status)
         client.status_changed.connect(self._client_status_changed)
 
-    def _change_box_text_with_status(self, status: int):
+    def _change_box_text_with_status(self, status: ray.ClientStatus):
         can_switch = ':switch:' in self.client.capabilities
 
         if status in (
                 ray.ClientStatus.STOPPED, ray.ClientStatus.PRECOPY,
                 ray.ClientStatus.QUIT, ray.ClientStatus.LOSE):
             text = ''
-        elif status == ray.ClientStatus.READY and can_switch:
+        elif status is ray.ClientStatus.READY and can_switch:
             text = _translate(
                 'id_renaming', 'The client project will be reload')
         else:
@@ -986,8 +1006,8 @@ class ClientRenameDialog(ChildDialog):
         
         self.ui.checkBoxIdRename.setText(full_text)    
 
-    def _client_status_changed(self, status: int):
-        if status == ray.ClientStatus.REMOVED:
+    def _client_status_changed(self, status: ray.ClientStatus):
+        if status is ray.ClientStatus.REMOVED:
             self.reject()
             
         self._change_box_text_with_status(status)
@@ -1009,14 +1029,20 @@ class ClientRenameDialog(ChildDialog):
         session = self.client.session
         ok = True
         
-        for cl in session.client_list + session.trashed_clients:
-            cl: Client
+        for cl in session.client_list:
             if cl.client_id == out_id:
                 self.ui.buttonBox.button(
-                    QDialogButtonBox.Ok).setEnabled(False)
+                    QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
                 return
         
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
+        for cl in session.trashed_clients:
+            if cl.client_id == out_id:
+                self.ui.buttonBox.button(
+                    QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
+                return
+        
+        self.ui.buttonBox.button(
+            QDialogButtonBox.StandardButton.Ok).setEnabled(True) # type:ignore
     
     def is_identifiant_renamed(self) -> bool:
         return self.ui.checkBoxIdRename.isChecked()
@@ -1032,11 +1058,11 @@ class SnapShotProgressDialog(ChildDialog):
         self.ui.setupUi(self)
         self.signaler.server_progress.connect(self.server_progress)
 
-    def _server_status_changed(self, server_status):
+    def _server_status_changed(self, server_status: ray.ServerStatus):
         self.close()
 
-    def server_progress(self, value):
-        self.ui.progressBar.setValue(value * 100)
+    def server_progress(self, value: float):
+        self.ui.progressBar.setValue(int(value * 100))
 
 
 class ScriptInfoDialog(ChildDialog):
@@ -1065,21 +1091,23 @@ class ScriptUserActionDialog(ChildDialog):
         self._is_terminated = False
 
     def _validate(self):
-        self.to_daemon('/reply', '/ray/gui/script_user_action',
+        self.to_daemon(osc_paths.REPLY, rg.SCRIPT_USER_ACTION,
                       'Dialog window validated')
         self._is_terminated = True
         self.accept()
 
     def _abort(self):
-        self.to_daemon('/error', '/ray/gui/script_user_action',
+        self.to_daemon(osc_paths.ERROR, rg.SCRIPT_USER_ACTION,
                       ray.Err.ABORT_ORDERED, 'Script user action aborted!')
         self._is_terminated = True
         self.accept()
 
     def _button_box_clicked(self, button):
-        if button == self.ui.buttonBox.button(QDialogButtonBox.Yes):
+        if button == self.ui.buttonBox.button(
+                QDialogButtonBox.StandardButton.Yes): # type:ignore
             self._validate()
-        elif button == self.ui.buttonBox.button(QDialogButtonBox.Ignore):
+        elif button == self.ui.buttonBox.button(
+                QDialogButtonBox.StandardButton.Ignore): # type:ignore
             self._abort()
 
     def set_main_text(self, text: str):
@@ -1163,13 +1191,13 @@ class DaemonUrlWindow(ChildDialog):
             error_text = _translate("url window", "<p align=\"left\">To run a network session,<br>open a terminal on another computer of this network.<br>Launch ray-daemon on port 1234 (for example)<br>by typing the command :</p><p align=\"left\"><code>ray-daemon -p 1234</code></p><p align=\"left\">Then paste below the first url<br>that ray-daemon gives you at startup.</p><p></p>")
 
         self.ui.labelError.setText(error_text)
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        self.ui.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
 
         self.tried_urls = ray.get_list_in_settings(RS.settings, 'network/tried_urls')
         last_tried_url = RS.settings.value('network/last_tried_url', '', type=str)
 
         self._completer = QCompleter(self.tried_urls)
-        self.ui.lineEdit.setCompleter(self._completer)
+        self.ui.lineEdit.setCompleter(self._completer) # type:ignore
 
         if ex_url:
             self.ui.lineEdit.setText(ex_url)
@@ -1179,18 +1207,19 @@ class DaemonUrlWindow(ChildDialog):
     def _allow_url(self, text: str):
         if not text:
             self.ui.lineEdit.completer().complete()
-            self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+            self.ui.buttonBox.button(
+                QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
             return
 
         if not text.startswith('osc.udp://'):
-            self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+            self.ui.buttonBox.button(
+                QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
             return
 
-        try:
-            addr = ray.get_liblo_address(text)
-            self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
-        except BaseException:
-            self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        addr = verified_address(text)
+        self.ui.buttonBox.button(
+                QDialogButtonBox.StandardButton.Ok).setEnabled( # type:ignore
+                    isinstance(addr, Address))
 
     def get_url(self):
         return self.ui.lineEdit.text()
@@ -1204,23 +1233,23 @@ class WaitingCloseUserDialog(ChildDialog):
 
         if is_dark_theme(self):
             self.ui.labelSaveIcon.setPixmap(
-                QPixmap(':scalable/breeze-dark/document-nosave.svg'))
+                QPixmap(':scalable/breeze-dark/document-nosave.svg')) # type:ignore
 
-        self.ui.pushButtonOk.setFocus(True)
+        self.ui.pushButtonOk.setFocus()
         self.ui.pushButtonUndo.clicked.connect(self._undo_close)
         self.ui.pushButtonSkip.clicked.connect(self._skip)
         self.ui.checkBox.setChecked(not RS.is_hidden(RS.HD_WaitCloseUser))
         self.ui.checkBox.clicked.connect(self._check_box_clicked)
 
-    def _server_status_changed(self, server_status):
-        if server_status != ray.ServerStatus.WAIT_USER:
+    def _server_status_changed(self, server_status: ray.ServerStatus):
+        if server_status is not ray.ServerStatus.WAIT_USER:
             self.accept()
 
     def _undo_close(self):
-        self.to_daemon('/ray/session/cancel_close')
+        self.to_daemon(r.session.CANCEL_CLOSE)
 
     def _skip(self):
-        self.to_daemon('/ray/session/skip_wait_user')
+        self.to_daemon(r.session.SKIP_WAIT_USER)
 
     def _check_box_clicked(self, state):
         RS.set_hidden(RS.HD_WaitCloseUser, bool(state))
@@ -1232,11 +1261,21 @@ class DonationsDialog(ChildDialog):
         self.ui = ui.donations.Ui_Dialog()
         self.ui.setupUi(self)
 
+        dark = '-dark' if is_dark_theme(self) else ''
+        self.ui.toolButtonImage.setIcon(
+            QIcon(f':scalable/breeze{dark}/handshake-deal.svg'))
+
+        self.ui.toolButtonDonate.clicked.connect(self._donate)
+
         self.ui.checkBox.setVisible(display_no_again)
         self.ui.checkBox.clicked.connect(self._check_box_clicked)
 
     def _check_box_clicked(self, state):
         RS.set_hidden(RS.HD_Donations, state)
+        
+    def _donate(self):
+        QDesktopServices.openUrl(
+            QUrl('https://liberapay.com/Houston4444'))
 
 
 class SystrayCloseDialog(ChildDialog):
@@ -1267,8 +1306,8 @@ class StartupDialog(ChildDialog):
         for recent_session in self.session.recent_sessions:
             session_item = QListWidgetItem(recent_session.replace('/', ' / '),
                                            self.ui.listWidgetRecentSessions)
-            session_item.setData(Qt.UserRole, recent_session)
-            self.ui.listWidgetRecentSessions.addItem(session_item)
+            session_item.setData(Qt.ItemDataRole.UserRole, recent_session)
+            self.ui.listWidgetRecentSessions.addItem(session_item) # type:ignore
 
         self.ui.listWidgetRecentSessions.setMinimumHeight(
             30 * len(self.session.recent_sessions))
@@ -1287,10 +1326,11 @@ class StartupDialog(ChildDialog):
         self.ui.pushButtonOpenSession.focus_on_new.connect(
             self._focus_on_new)
 
-        self.ui.listWidgetRecentSessions.setFocus(Qt.OtherFocusReason)
+        self.ui.listWidgetRecentSessions.setFocus(
+            Qt.FocusReason.OtherFocusReason) # type:ignore
 
-    def _server_status_changed(self, server_status):
-        if server_status != ray.ServerStatus.OFF:
+    def _server_status_changed(self, server_status: ray.ServerStatus):
+        if server_status is not ray.ServerStatus.OFF:
             self.reject()
 
     def _new_session_clicked(self):
@@ -1302,13 +1342,14 @@ class StartupDialog(ChildDialog):
         self.reject()
 
     def _focus_on_list(self):
-        self.ui.listWidgetRecentSessions.setFocus(Qt.OtherFocusReason)
+        self.ui.listWidgetRecentSessions.setFocus(
+            Qt.FocusReason.OtherFocusReason) # type:ignore
 
     def _focus_on_new(self):
-        self.ui.pushButtonNewSession.setFocus(Qt.OtherFocusReason)
+        self.ui.pushButtonNewSession.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def _focus_on_open(self):
-        self.ui.pushButtonOpenSession.setFocus(Qt.OtherFocusReason)
+        self.ui.pushButtonOpenSession.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def not_again_value(self)->bool:
         return not self.ui.checkBox.isChecked()
@@ -1316,24 +1357,25 @@ class StartupDialog(ChildDialog):
     def get_selected_session(self)->str:
         current_item = self.ui.listWidgetRecentSessions.currentItem()
         if current_item:
-            return current_item.data(Qt.UserRole)
+            return current_item.data(Qt.ItemDataRole.UserRole)
         return ''
 
     def get_clicked_action(self)->int:
         return self._clicked_action
 
     def keyPressEvent(self, event: QKeyEvent):
-        if event.key() == Qt.Key_Left:
-            self.ui.pushButtonNewSession.setFocus(Qt.OtherFocusReason)
-        elif event.key() == Qt.Key_Right:
-            self.ui.pushButtonOpenSession.setFocus(Qt.OtherFocusReason)
-        elif event.key() in (Qt.Key_Up, Qt.Key_Down):
-            self.ui.listWidgetRecentSessions.setFocus(Qt.OtherFocusReason)
+        if event.key() == Qt.Key.Key_Left:
+            self.ui.pushButtonNewSession.setFocus(Qt.FocusReason.OtherFocusReason)
+        elif event.key() == Qt.Key.Key_Right:
+            self.ui.pushButtonOpenSession.setFocus(Qt.FocusReason.OtherFocusReason)
+        elif event.key() in (Qt.Key.Key_Up, Qt.Key.Key_Down):
+            self.ui.listWidgetRecentSessions.setFocus(
+                Qt.FocusReason.OtherFocusReason) # type:ignore
 
-        if QApplication.keyboardModifiers() & Qt.ControlModifier:
-            if event.key() == Qt.Key_N:
+        if QApplication.keyboardModifiers() & Qt.KeyboardModifier.ControlModifier:
+            if event.key() == Qt.Key.Key_N:
                 self._new_session_clicked()
-            elif event.key() == Qt.Key_O:
+            elif event.key() == Qt.Key.Key_O:
                 self._open_session_clicked()
 
         ChildDialog.keyPressEvent(self, event)
