@@ -1,26 +1,34 @@
+
+# Imports from standard library
 import os
 import signal
 from typing import TYPE_CHECKING
 
-from PyQt5.QtCore import QTimer, QFile
-from PyQt5.QtGui import QShowEvent
-from PyQt5.QtWidgets import QFileDialog, QFrame
+# third party imports
+from qtpy.QtCore import QTimer, QFile
+from qtpy.QtGui import QShowEvent
+from qtpy.QtWidgets import QFileDialog, QFrame
 
+# Imports from src/shared
 import ray
+from osclib import is_valid_osc_url
+import osc_paths.ray as r
 
+# Local imports
 from gui_tools import _translate, client_status_string, get_app_icon
 from child_dialogs import ChildDialog
 from client_prop_adv_dialog import AdvancedPropertiesDialog
 
-if TYPE_CHECKING:
-    from gui_client import Client
-
+# Import UIs made with Qt-Designer
 import ui.ray_hack_copy
 import ui.client_properties
 import ui.nsm_properties
 import ui.ray_hack_properties
 import ui.ray_net_properties
 
+if TYPE_CHECKING:
+    from gui_client import Client
+
 
 class RayHackCopyDialog(ChildDialog):
     def __init__(self, parent):
@@ -43,7 +51,7 @@ class RayHackCopyDialog(ChildDialog):
 
 
 class ClientPropertiesDialog(ChildDialog):
-    def __init__(self, parent, client: 'Client'):
+    def __init__(self, parent, client: ray.ClientData):
         ChildDialog.__init__(self, parent)
         self.ui = ui.client_properties.Ui_Dialog()
         self.ui.setupUi(self)
@@ -64,7 +72,7 @@ class ClientPropertiesDialog(ChildDialog):
         self.ui.tabWidget.setCurrentIndex(0)
         
         self._advanced_dialog = AdvancedPropertiesDialog(
-            self.parentWidget(), client)
+            self.parentWidget(), client) # type:ignore
 
     def _change_icon_with_text(self, text: str):
         icon = get_app_icon(text, self)
@@ -95,6 +103,9 @@ class ClientPropertiesDialog(ChildDialog):
         self.client.check_last_save = self.ui.checkBoxSaveStop.isChecked()
         self.client.ignored_extensions = \
             self.ui.lineEditIgnoredExtensions.text()
+        
+        if TYPE_CHECKING:
+            assert isinstance(self.client, Client)
         self.client.send_properties_to_daemon()
 
         # better for user to wait a little before close the window
@@ -105,12 +116,13 @@ class ClientPropertiesDialog(ChildDialog):
 
     @staticmethod
     def create(window, client: ray.ClientData) -> 'ClientPropertiesDialog':
-        if client.protocol == ray.Protocol.NSM:
-            return NsmClientPropertiesDialog(window, client)
-        if client.protocol == ray.Protocol.RAY_HACK:
-            return RayHackClientPropertiesDialog(window, client)
-        if client.protocol == ray.Protocol.RAY_NET:
-            return RayNetClientPropertiesDialog(window, client)
+        match client.protocol:
+            case ray.Protocol.NSM | ray.Protocol.INTERNAL:
+                return NsmClientPropertiesDialog(window, client)
+            case ray.Protocol.RAY_HACK:
+                return RayHackClientPropertiesDialog(window, client)
+            case ray.Protocol.RAY_NET:
+                return RayNetClientPropertiesDialog(window, client)
 
         return ClientPropertiesDialog(window, client)
 
@@ -131,12 +143,15 @@ class ClientPropertiesDialog(ChildDialog):
     def set_on_second_tab(self):
         self.ui.tabWidget.setCurrentIndex(1)
 
-    def update_status(self, status):
-        pass
+    def update_status(self, status: ray.ClientStatus):
+        ...
+
+    def enable_test_zone(self, yesno: bool):
+        ...
 
     def update_contents(self):
         self.ui.labelId.setText(self.client.client_id)
-        self.ui.labelProtocol.setText(ray.protocol_to_str(self.client.protocol))
+        self.ui.labelProtocol.setText(self.client.protocol.to_string())
         self.ui.lineEditIcon.setText(self.client.icon)
         self.ui.lineEditLabel.setText(self.client.label)
         self.ui.plainTextEditDescription.setPlainText(self.client.description)
@@ -152,12 +167,19 @@ class ClientPropertiesDialog(ChildDialog):
 class NsmClientPropertiesDialog(ClientPropertiesDialog):
     def __init__(self, parent, client):
         ClientPropertiesDialog.__init__(self, parent, client)
+        
         self.nsmui_frame = QFrame()
         self.nsmui = ui.nsm_properties.Ui_Frame()
         self.nsmui.setupUi(self.nsmui_frame)
-        self.ui.verticalLayoutProtocol.addWidget(self.nsmui_frame)
+        self.ui.verticalLayoutProtocol.addWidget(
+            self.nsmui_frame) # type:ignore
 
-        self.ui.tabWidget.setTabText(1, 'NSM')
+        if self.client.protocol is ray.Protocol.INTERNAL:
+            self.ui.tabWidget.setTabText(1, 'Internal')
+            self.nsmui.groupBoxEnv.setVisible(False)
+            self.nsmui.checkBoxTerminal.setVisible(False)
+        else:
+            self.ui.tabWidget.setTabText(1, 'NSM')
 
     def _save_changes(self):
         self.client.executable_path = self.nsmui.lineEditExecutable.text()
@@ -203,7 +225,8 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
 
         self.config_file = ""
 
-        self.ui.verticalLayoutProtocol.addWidget(self.ray_hack_frame)
+        self.ui.verticalLayoutProtocol.addWidget(
+            self.ray_hack_frame) # type:ignore
 
         self.ui.tabWidget.setTabText(1, 'Ray-Hack')
         self.rhack.labelWorkingDir.setText(self._get_work_dir_base())
@@ -256,6 +279,9 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         self.rhack.toolButtonIcon.setIcon(icon)
 
     def _save_changes(self):
+        if self.client.ray_hack is None:
+            return
+
         self.client.ray_hack.config_file = self.rhack.lineEditConfigFile.text()
         self.client.ray_hack.save_sig = self.rhack.comboSaveSig.currentData()
         self.client.ray_hack.stop_sig = self.rhack.comboStopSig.currentData()
@@ -263,16 +289,14 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
             self.rhack.checkBoxWaitWindow.isChecked()
 
         no_save_level = 0
-
-        if self.rhack.checkBoxCloseGracefully.isChecked():
-            no_save_level = 2
-        elif self.rhack.checkBoxTellUser.isChecked():
+        if self.rhack.checkBoxTellUser.isChecked():
             no_save_level = 1
+            if self.rhack.checkBoxCloseGracefully.isChecked():
+                no_save_level = 2
 
         self.client.ray_hack.no_save_level = no_save_level
-        #self.client.ray_hack.close_gracefully = \
-            #self.rhack.checkBoxCloseGracefully.isChecked()
-
+        if TYPE_CHECKING:
+            assert isinstance(self.client, Client)
         self.client.send_ray_hack()
 
         self.client.executable_path = self.rhack.lineEditExecutable.text()
@@ -280,18 +304,18 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         self.client.pre_env = self.rhack.lineEditEnviron.text()
         ClientPropertiesDialog._save_changes(self)
 
-    def _get_work_dir_base(self)->str:
+    def _get_work_dir_base(self) -> str:
         prefix = self.session.name
-        if self.client.prefix_mode == ray.PrefixMode.CLIENT_NAME:
+        if self.client.prefix_mode is ray.PrefixMode.CLIENT_NAME:
             prefix = self.client.name
-        elif self.client.prefix_mode == ray.PrefixMode.CUSTOM:
+        elif self.client.prefix_mode is ray.PrefixMode.CUSTOM:
             prefix = self.client.custom_prefix
-
-        return "%s.%s" % (prefix, self.client.client_id)
+        
+        return f"{prefix}.{self.client.client_id}"
 
     def _browse_config_file(self):
         work_dir_base = self._get_work_dir_base()
-        work_dir = "%s/%s" % (self.session.path, work_dir_base)
+        work_dir = f"{self.session.path}/{work_dir_base}"
 
         config_file, ok = QFileDialog.getOpenFileName(
             self,
@@ -312,9 +336,8 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
                     if copy_dialog.rename_file:
                         base, pt, extension = os.path.basename(
                             config_file).rpartition('.')
-
-                        config_file = "%s.%s" % (self.session.name,
-                                                 extension)
+                        
+                        config_file = f"{self.session.name}.{extension}"
                         if not base:
                             config_file = self.session.name
                     else:
@@ -326,9 +349,9 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
 
         if (self.session.name
                 and (self.config_file == self.session.name
-                     or self.config_file.startswith("%s." % self.session.name))):
-            self.config_file = self.config_file.replace(self.session.name,
-                                                        "$RAY_SESSION_NAME")
+                     or self.config_file.startswith(f'{self.session.name}.'))):
+            self.config_file = self.config_file.replace(
+                self.session.name, "$RAY_SESSION_NAME")
         self.rhack.lineEditConfigFile.setText(self.config_file)
 
     def _is_allowed(self):
@@ -345,7 +368,7 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
 
         self.rhack.pushButtonStart.setEnabled(
             bool(self._acceptable_arguments
-                 and self._current_status == ray.ClientStatus.STOPPED))
+                 and self._current_status is ray.ClientStatus.STOPPED))
         self.ui.pushButtonSaveChanges.setEnabled(self._is_allowed())
 
     def _line_edit_config_file_changed(self, text):
@@ -359,6 +382,9 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
             bool(text and self.rhack.comboSaveSig.currentData() == 0))
 
     def _start_client(self):
+        if self.client.ray_hack is None:
+            return
+        
         executable = self.client.executable_path
         arguments = self.client.arguments
         config_file = self.client.ray_hack.config_file
@@ -367,22 +393,18 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         self.client.arguments = self.rhack.lineEditArguments.text()
         self.client.ray_hack.config_file = self.rhack.lineEditConfigFile.text()
 
+        if TYPE_CHECKING:
+            assert isinstance(self.client, Client)
         self.client.send_properties_to_daemon()
-        self.to_daemon('/ray/client/resume', self.client.client_id)
-
-        self.client.executable_path = executable
-        self.client.arguments = arguments
-        self.client.ray_hack.config_file = config_file
-
-        self.client.send_properties_to_daemon()
+        self.to_daemon(r.client.RESUME, self.client.client_id)
 
     def _stop_client(self):
-        self.to_daemon('/ray/client/send_signal', self.client.client_id,
-                      self.rhack.comboStopSig.currentData())
+        self.to_daemon(r.client.SEND_SIGNAL, self.client.client_id,
+                       self.rhack.comboStopSig.currentData())
 
     def _save_client(self):
-        self.to_daemon('/ray/client/send_signal', self.client.client_id,
-                      self.rhack.comboSaveSig.currentData())
+        self.to_daemon(r.client.SEND_SIGNAL, self.client.client_id,
+                       self.rhack.comboSaveSig.currentData())
 
     def lock_widgets(self):
         ClientPropertiesDialog.lock_widgets(self)
@@ -403,33 +425,35 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         self.rhack.labelWorkingDirTitle.setVisible(False)
         self.rhack.labelWorkingDir.setVisible(False)
 
-    def update_status(self, status):
+    def update_status(self, status: ray.ClientStatus):
         self._current_status = status
         self.rhack.lineEditClientStatus.setText(client_status_string(status))
 
-        if status in (ray.ClientStatus.LAUNCH,
-                      ray.ClientStatus.OPEN,
-                      ray.ClientStatus.SWITCH,
-                      ray.ClientStatus.NOOP):
-            self.rhack.pushButtonStart.setEnabled(False)
-            self.rhack.pushButtonStop.setEnabled(True)
-            self.rhack.pushButtonSave.setEnabled(False)
-        elif status == ray.ClientStatus.READY:
-            self.rhack.pushButtonStart.setEnabled(False)
-            self.rhack.pushButtonStop.setEnabled(True)
-            self.rhack.pushButtonSave.setEnabled(
-                bool(self.rhack.comboSaveSig.currentData() != 0))
-        elif status == ray.ClientStatus.STOPPED:
-            self.rhack.pushButtonStart.setEnabled(self._is_allowed())
-            self.rhack.pushButtonStop.setEnabled(False)
-            self.rhack.pushButtonSave.setEnabled(False)
-        elif status == ray.ClientStatus.PRECOPY:
-            self.rhack.pushButtonStart.setEnabled(False)
-            self.rhack.pushButtonStart.setEnabled(False)
-            self.rhack.pushButtonSave.setEnabled(False)
+        match status:
+            case ray.ClientStatus.LAUNCH | ray.ClientStatus.OPEN \
+                    | ray.ClientStatus.SWITCH | ray.ClientStatus.NOOP:
+                self.rhack.pushButtonStart.setEnabled(False)
+                self.rhack.pushButtonStop.setEnabled(True)
+                self.rhack.pushButtonSave.setEnabled(False)
+            case ray.ClientStatus.READY:
+                self.rhack.pushButtonStart.setEnabled(False)
+                self.rhack.pushButtonStop.setEnabled(True)
+                self.rhack.pushButtonSave.setEnabled(
+                    bool(self.rhack.comboSaveSig.currentData() != 0))
+            case ray.ClientStatus.STOPPED:
+                self.rhack.pushButtonStart.setEnabled(self._is_allowed())
+                self.rhack.pushButtonStop.setEnabled(False)
+                self.rhack.pushButtonSave.setEnabled(False)
+            case ray.ClientStatus.PRECOPY:
+                self.rhack.pushButtonStart.setEnabled(False)
+                self.rhack.pushButtonStart.setEnabled(False)
+                self.rhack.pushButtonSave.setEnabled(False)
 
     def update_contents(self):
         ClientPropertiesDialog.update_contents(self)
+        if self.client.ray_hack is None:
+            return
+        
         self.rhack.lineEditExecutable.setText(self.client.executable_path)
         self.rhack.lineEditArguments.setText(self.client.arguments)
         self.rhack.lineEditEnviron.setText(self.client.pre_env)
@@ -438,6 +462,7 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
 
         save_sig = self.client.ray_hack.save_sig
 
+        i = 0
         for i in range(self.rhack.comboSaveSig.count()):
             if self.rhack.comboSaveSig.itemData(i) == save_sig:
                 self.rhack.comboSaveSig.setCurrentIndex(i)
@@ -447,7 +472,7 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
                 signal_text = str(
                     signal.Signals(save_sig)).rpartition('.')[2]
                 self.rhack.comboSaveSig.addItem(signal_text, save_sig)
-                self.rhack.comboSaveSig.setCurrentIndex(i+1)
+                self.rhack.comboSaveSig.setCurrentIndex(i + 1)
             except:
                 self.rhack.comboSaveSig.setCurrentIndex(0)
 
@@ -473,7 +498,6 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         self.rhack.checkBoxCloseGracefully.setChecked(
             bool(self.client.ray_hack.no_save_level == 2))
 
-
     def enable_test_zone(self, yesno: bool):
         self.rhack.groupBoxTestZone.setChecked(yesno)
 
@@ -498,7 +522,8 @@ class RayNetClientPropertiesDialog(ClientPropertiesDialog):
 
         self.ui.groupBoxSnapshots.setVisible(False)
 
-        self.ui.verticalLayoutProtocol.addWidget(self.ray_net_frame)
+        self.ui.verticalLayoutProtocol.addWidget(
+            self.ray_net_frame) # type:ignore
         self.ui.tabWidget.setTabText(1, 'Ray-Net')
 
     def lock_widgets(self):
@@ -508,6 +533,9 @@ class RayNetClientPropertiesDialog(ClientPropertiesDialog):
         self.rnet.lineEditTemplate.setReadOnly(True)
 
     def update_contents(self):
+        if self.client.ray_net is None:
+            return
+        
         ClientPropertiesDialog.update_contents(self)
         self.rnet.labelClientName.setText(self.client.name)
         self.rnet.labelCapabilities.setText(self._get_capacities_line())
@@ -516,17 +544,22 @@ class RayNetClientPropertiesDialog(ClientPropertiesDialog):
         self.rnet.lineEditTemplate.setText(self.client.ray_net.session_template)
 
     def _save_changes(self):
+        if self.client.ray_net is None:
+            return
+        
         new_url = self.rnet.lineEditDaemonUrl.text()
         new_root = self.rnet.lineEditSessionRoot.text()
         new_template = self.rnet.lineEditTemplate.text()
 
-        if ray.is_valid_osc_url(new_url):
+        if is_valid_osc_url(new_url):
             self.client.ray_net.daemon_url = new_url
         if ray.is_valid_full_path(new_root):
             self.client.ray_net.session_root = new_root
         if '/' not in new_template:
             self.client.ray_net.session_template = new_template
 
+        if TYPE_CHECKING:
+            assert isinstance(self.client, Client)
         self.client.send_ray_net()
         ClientPropertiesDialog._save_changes(self)
 
