@@ -1,10 +1,21 @@
-import time
-import sys
-from typing import TYPE_CHECKING, Optional
-from PyQt5.QtCore import QObject, pyqtSignal
-from PyQt5.QtWidgets import QAction
 
+# Imports from standard library
+import time
+import logging
+from typing import TYPE_CHECKING, Optional
+
+# third party imports
+from qtpy.QtCore import QObject, Signal # type:ignore
+if TYPE_CHECKING:
+    # FIX : QAction not found by pylance
+    from qtpy.QtGui import QAction
+from qtpy.QtWidgets import QAction # type:ignore
+
+# Imports from src/shared
 import ray
+import osc_paths.ray as r
+
+# Local imports
 from gui_server_thread import GuiServerThread
 from client_properties_dialog import ClientPropertiesDialog
 
@@ -12,11 +23,14 @@ if TYPE_CHECKING:
     from gui_session import SignaledSession
 
 
+_logger = logging.getLogger(__name__)
+
+
 class Client(QObject, ray.ClientData):
-    status_changed = pyqtSignal(int)
+    status_changed = Signal(ray.ClientStatus)
 
     def __init__(self, session: 'SignaledSession',
-                 client_id: str, protocol: int):
+                 client_id: str, protocol_int: int):
         QObject.__init__(self)
 
         self.session = session
@@ -24,7 +38,7 @@ class Client(QObject, ray.ClientData):
 
         # set ClientData attributes
         self.client_id = client_id
-        self.protocol = protocol
+        self.protocol = ray.Protocol(protocol_int)
         self.ray_hack = ray.RayHack()
         self.ray_net = ray.RayNet()
         
@@ -38,15 +52,15 @@ class Client(QObject, ray.ClientData):
         self.no_save_level = 0
         self.last_save = time.time()
         self.widget = self.main_win.create_client_widget(self)
-        self._properties_dialog: ClientPropertiesDialog = None
+        self._properties_dialog: Optional[ClientPropertiesDialog] = None
 
-    def set_status(self, status: int):
+    def set_status(self, status: ray.ClientStatus):
         self._previous_status = self.status
         self.status = status
         self.status_changed.emit(status)
 
         if (not self.has_dirty
-                and self.status == ray.ClientStatus.READY
+                and self.status is ray.ClientStatus.READY
                 and self._previous_status in (
                     ray.ClientStatus.OPEN, ray.ClientStatus.SAVE)):
             self.last_save = time.time()
@@ -70,10 +84,6 @@ class Client(QObject, ray.ClientData):
         self.dirty_state = dirty
         self.widget.set_dirty_state(dirty)
 
-    def set_no_save_level(self, no_save_level: int):
-        self.no_save_level = no_save_level
-        self.widget.set_no_save_level(no_save_level)
-
     def set_progress(self, progress: float):
         self.widget.set_progress(progress)
 
@@ -88,45 +98,51 @@ class Client(QObject, ray.ClientData):
         self.widget.update_client_data()
 
     def update_ray_hack(self, *args):
+        if self.ray_hack is None:
+            return
+        
         self.ray_hack.update(*args)
         self.widget.update_client_data()
 
     def update_ray_net(self, *args):
+        if self.ray_net is None:
+            return
+        
         self.ray_net.update(*args)
         self.widget.update_client_data()
 
     def send_properties_to_daemon(self):
         server = GuiServerThread.instance()
-        if not server:
-            sys.stderr.write(
-                'Server not found. Client %s can not send its properties\n'
-                % self.client_id)
+        if server is None:
+            _logger.error(
+                'Server not found. '
+                f'Client {self.client_id} can not send its properties')
             return
 
-        server.to_daemon('/ray/client/update_properties',
+        server.to_daemon(r.client.UPDATE_PROPERTIES,
                          *ray.ClientData.spread_client(self))
 
     def send_ray_hack(self):
-        if self.protocol != ray.Protocol.RAY_HACK:
+        if self.ray_hack is None:
             return
 
         server = GuiServerThread.instance()
-        if not server:
+        if server is None:
             return
 
-        server.to_daemon('/ray/client/update_ray_hack_properties',
+        server.to_daemon(r.client.UPDATE_RAY_HACK_PROPERTIES,
                          self.client_id,
                          *self.ray_hack.spread())
 
     def send_ray_net(self):
-        if self.protocol != ray.Protocol.RAY_NET:
+        if self.ray_net is None:
             return
 
         server = GuiServerThread.instance()
-        if not server:
+        if server is None:
             return
 
-        server.to_daemon('/ray/client/update_ray_net_properties',
+        server.to_daemon(r.client.UPDATE_RAY_NET_PROPERTIES,
                          self.client_id,
                          *self.ray_net.spread())
 
@@ -134,13 +150,17 @@ class Client(QObject, ray.ClientData):
         if self._properties_dialog is None:
             self._properties_dialog = ClientPropertiesDialog.create(
                 self.main_win, self)
+
         self._properties_dialog.update_contents()
+
         if second_tab:
-            if self.protocol == ray.Protocol.RAY_HACK:
+            if self.protocol is ray.Protocol.RAY_HACK:
                 self._properties_dialog.enable_test_zone(True)
             self._properties_dialog.set_on_second_tab()
+
         self._properties_dialog.show()
-        if ray.get_window_manager() != ray.WindowManager.WAYLAND:
+
+        if ray.get_window_manager() is not ray.WindowManager.WAYLAND:
             self._properties_dialog.activateWindow()
 
     def close_properties_dialog(self):
@@ -158,36 +178,18 @@ class Client(QObject, ray.ClientData):
             self.set_gui_enabled()
 
     # method not used yet
-    def get_project_path(self)->str:
+    def get_project_path(self) -> str:
         if not self.session.path:
             return ''
 
         prefix = self.session.name
 
-        if self.prefix_mode == ray.PrefixMode.CLIENT_NAME:
+        if self.prefix_mode is ray.PrefixMode.CLIENT_NAME:
             prefix = self.name
-        elif self.prefix_mode == ray.PrefixMode.CUSTOM:
+        elif self.prefix_mode is ray.PrefixMode.CUSTOM:
             prefix = self.custom_prefix
 
-        return "%s/%s.%s" % (self.session.path, prefix, self.client_id)
-
-    # method not used yet
-    def get_icon_search_path(self) -> list[str]:
-        if not self.session.daemon_manager.is_local:
-            return []
-
-        project_path = self.get_project_path()
-        if not project_path:
-            return []
-
-        search_list = []
-        main_icon_path = '.local/share/icons'
-        search_list.append("%s/%s" % (search_list, main_icon_path))
-
-        for path in ('16x16', '24x24', '32x32', '64x64', 'scalable'):
-            search_list.append("%s/%s/%s" % (project_path,
-                                             main_icon_path, path))
-        return search_list
+        return f'{self.session.path}/{prefix}.{self.client_id}'
 
 
 class TrashedClient(ray.ClientData):
