@@ -1,28 +1,36 @@
 
+
+# Imports from standard library
 from typing import TYPE_CHECKING
-from PyQt5.QtWidgets import (QListWidget, QListWidgetItem,
-                             QFrame, QMenu, QBoxLayout)
-from PyQt5.QtGui import (QIcon, QPixmap, QFontMetrics, QContextMenuEvent,
-                         QMouseEvent, QKeyEvent)
-from PyQt5.QtCore import pyqtSlot, QSize, Qt, pyqtSignal
 
+# third party imports
+from qtpy.QtWidgets import (QListWidget, QListWidgetItem,
+                             QFrame, QMenu, QBoxLayout)
+from qtpy.QtGui import (QIcon, QFontMetrics, QContextMenuEvent,
+                         QMouseEvent, QKeyEvent)
+from qtpy.QtCore import Slot, QSize, Qt, Signal # type:ignore
+
+# Imports from src/shared
 import ray
+import osc_paths.ray as r
+
+# Local imports
 import child_dialogs
 import snapshots_dialog
-
 from gui_server_thread import GuiServerThread
 from gui_tools import (client_status_string, _translate, is_dark_theme,
-                       RayIcon, split_in_two, get_app_icon)
+                       ray_icon, split_in_two, get_app_icon)
+
+# Import UIs made with Qt-Designer
+import ui.client_slot
 
 if TYPE_CHECKING:
     from gui_client import Client
     from gui_session import Session
 
-import ui.client_slot
-
 
 class ClientSlot(QFrame):
-    clicked = pyqtSignal(str)
+    clicked = Signal(str)
     
     def __init__(self, list_widget: 'ListWidgetClients',
                  list_widget_item: 'ClientItem', client: 'Client'):
@@ -41,7 +49,7 @@ class ClientSlot(QFrame):
         self._icon_off = QIcon()
 
         self.ui.toolButtonGUI.setVisible(False)
-        if client.protocol != ray.Protocol.RAY_HACK:
+        if client.protocol is not ray.Protocol.RAY_HACK:
             self.ui.toolButtonHack.setVisible(False)
 
         # connect buttons to functions
@@ -66,32 +74,41 @@ class ClientSlot(QFrame):
 
         self._menu = QMenu(self)
 
-        self._menu.addAction(self.ui.actionSaveAsApplicationTemplate)
-        self._menu.addAction(self.ui.actionRename)
-        self._menu.addAction(self.ui.actionReturnToAPreviousState)
-        self._menu.addAction(self.ui.actionFindBoxesInPatchbay)
-        self._menu.addAction(self.ui.actionProperties)
+        self._menu.addAction(
+            self.ui.actionSaveAsApplicationTemplate) # type:ignore
+        self._menu.addAction(
+            self.ui.actionRename) # type:ignore
+        self._menu.addAction(
+            self.ui.actionReturnToAPreviousState) # type:ignore
+        self._menu.addAction(
+            self.ui.actionFindBoxesInPatchbay) # type:ignore
+        self._menu.addAction(
+            self.ui.actionProperties) # type:ignore
 
         self.ui.actionReturnToAPreviousState.setVisible(
             self.main_win.has_git)
 
-        self.ui.iconButton.setMenu(self._menu)
+        self.ui.iconButton.setMenu(self._menu) # type:ignore
         
         dark = is_dark_theme(self)
         
-        self._save_icon = RayIcon('document-save', dark)
-        self._saved_icon = RayIcon('document-saved', dark)
-        self._unsaved_icon = RayIcon('document-unsaved', dark)
-        self._no_save_icon = RayIcon('document-nosave', dark)
-        self._icon_visible = RayIcon('visibility', dark)
-        self._icon_invisible = RayIcon('hint', dark)
-        self._stop_icon = RayIcon('media-playback-stop', dark)
-        self._kill_icon = RayIcon('media-playback-stop_red', dark)
+        self._save_icon = ray_icon('document-save', dark)
+        self._saved_icon = ray_icon('document-saved', dark)
+        self._unsaved_icon = ray_icon('document-unsaved', dark)
+        self._no_save_icon = ray_icon('document-nosave', dark)
+        self._icon_visible = ray_icon('visibility', dark)
+        self._icon_invisible = ray_icon('hint', dark)
+        self._stop_icon = ray_icon('media-playback-stop', dark)
+        self._kill_icon = ray_icon('media-playback-stop_red', dark)
 
-        self.ui.startButton.setIcon(RayIcon('media-playback-start', dark))
-        self.ui.closeButton.setIcon(RayIcon('window-close', dark))
-        self.ui.saveButton.setIcon(self._save_icon)
-        self.ui.stopButton.setIcon(self._stop_icon)
+        self.ui.startButton.setIcon(
+            ray_icon('media-playback-start', dark)) # type:ignore
+        self.ui.closeButton.setIcon(
+            ray_icon('window-close', dark)) # type:ignore
+        self.ui.saveButton.setIcon(
+            self._save_icon) # type:ignore
+        self.ui.stopButton.setIcon(
+            self._stop_icon) # type:ignore
 
         if ':optional-gui:' in self.client.capabilities:
             self.set_gui_state(self.client.gui_state)
@@ -110,12 +127,12 @@ class ClientSlot(QFrame):
 
     def _change_gui_state(self):
         if self._gui_state:
-            self.to_daemon('/ray/client/hide_optional_gui', self.get_client_id())
+            self.to_daemon(r.client.HIDE_OPTIONAL_GUI, self.get_client_id())
         else:
-            self.to_daemon('/ray/client/show_optional_gui', self.get_client_id())
+            self.to_daemon(r.client.SHOW_OPTIONAL_GUI, self.get_client_id())
 
     def _order_hack_visibility(self, state):
-        if self.client.protocol != ray.Protocol.RAY_HACK:
+        if self.client.protocol is not ray.Protocol.RAY_HACK:
             return
 
         if state:
@@ -124,11 +141,11 @@ class ClientSlot(QFrame):
             self.client.close_properties_dialog()
 
     def _start_client(self):
-        self.to_daemon('/ray/client/resume', self.get_client_id())
+        self.to_daemon(r.client.RESUME, self.get_client_id())
 
     def _stop_client(self):
         if self._stop_is_kill:
-            self.to_daemon('/ray/client/kill', self.get_client_id())
+            self.to_daemon(r.client.KILL, self.get_client_id())
             return
 
         # we need to prevent accidental stop with a window confirmation
@@ -136,10 +153,10 @@ class ClientSlot(QFrame):
         self.main_win.stop_client(self.get_client_id())
 
     def _save_client(self):
-        self.to_daemon('/ray/client/save', self.get_client_id())
+        self.to_daemon(r.client.SAVE, self.get_client_id())
 
     def _trash_client(self):
-        self.to_daemon('/ray/client/trash', self.get_client_id())
+        self.to_daemon(r.client.TRASH, self.get_client_id())
 
     def _abort_copy(self):
         self.main_win.abort_copy_client(self.get_client_id())
@@ -152,7 +169,7 @@ class ClientSlot(QFrame):
             return
 
         template_name = dialog.get_template_name()
-        self.to_daemon('/ray/client/save_as_template',
+        self.to_daemon(r.client.SAVE_AS_TEMPLATE,
                        self.get_client_id(), template_name)
 
     def _open_snapshots_dialog(self):
@@ -161,7 +178,9 @@ class ClientSlot(QFrame):
         dialog.exec()
         if dialog.result():
             snapshot = dialog.get_selected_snapshot()
-            self.to_daemon('/ray/client/open_snapshot',
+            if snapshot is None:
+                return
+            self.to_daemon(r.client.OPEN_SNAPSHOT,
                           self.get_client_id(), snapshot)
 
     def _find_patchbay_boxes(self):
@@ -178,16 +197,9 @@ class ClientSlot(QFrame):
             
             if dialog.is_identifiant_renamed():
                 self.to_daemon(
-                    '/ray/client/full_rename',
+                    r.client.FULL_RENAME,
                     self.client.client_id,
                     label)
-                # self.to_daemon(
-                #     '/ray/client/change_advanced_properties',
-                #     self.client.client_id,
-                #     self.client.label.replace(' ', '_'),
-                #     int(self.client.prefix_mode),
-                #     self.client.custom_prefix,
-                #     int(ray.JackNaming.LONG))
                 return
 
             self.client.label = label
@@ -211,38 +223,40 @@ class ClientSlot(QFrame):
             self.ui.startButton.setVisible(True)
             self.ui.stopButton.setVisible(True)
             self.ui.toolButtonHack.setVisible(
-                self.client.protocol == ray.Protocol.RAY_HACK)
+                self.client.protocol is ray.Protocol.RAY_HACK)
 
     def _set_fat(self, yesno: bool, very_fat=False):
         if yesno:
-            self.ui.mainLayout.setDirection(QBoxLayout.TopToBottom)
+            self.ui.mainLayout.setDirection(
+                QBoxLayout.Direction.TopToBottom) # type:ignore
             self.ui.spacerLeftOfDown.setVisible(True)
             self.list_widget_item.setSizeHint(
                 QSize(100, 80 if very_fat else 70))
         else:
             self.ui.spacerLeftOfDown.setVisible(False)
-            self.ui.mainLayout.setDirection(QBoxLayout.LeftToRight)
+            self.ui.mainLayout.setDirection(
+                QBoxLayout.Direction.LeftToRight) # type:ignore
             self.list_widget_item.setSizeHint(QSize(100, 45))
 
     def _gray_icon(self, gray: bool):
         if gray:
-            self.ui.iconButton.setIcon(self._icon_off)
+            self.ui.iconButton.setIcon(self._icon_off) # type:ignore
         else:
-            self.ui.iconButton.setIcon(self._icon_on)
+            self.ui.iconButton.setIcon(self._icon_on) # type:ignore
 
     def get_client_id(self):
         return self.client.client_id
 
     def update_layout(self):
         font = self.ui.ClientName.font()
-        main_size = QFontMetrics(font).width(self.client.prettier_name())
+        main_size = QFontMetrics(font).horizontalAdvance(self.client.prettier_name())
 
         layout_width = self.list_widget.width()
 
         self._set_very_short(layout_width < 233)
 
         scroll_bar = self.list_widget.verticalScrollBar()
-        if scroll_bar.isVisible():
+        if scroll_bar is not None and scroll_bar.isVisible():
             layout_width -= scroll_bar.width()
 
         max_label_width = layout_width - 231
@@ -266,7 +280,7 @@ class ClientSlot(QFrame):
             if not text:
                 continue
 
-            size = QFontMetrics(font).width(text)
+            size = QFontMetrics(font).horizontalAdvance(text)
             max_size = max(max_size, size)
 
         if max_size <= max_label_width:
@@ -290,31 +304,39 @@ class ClientSlot(QFrame):
         self.ui.ClientName.setText('\n'.join((top, bottom)))
 
     def update_client_data(self):
-        # set main label and main disposition
+        # set main label and main layout
         self.update_layout()
 
         # set tool tip
-        tool_tip = "<html><head/><body>"
-        tool_tip += "<p><span style=\" font-weight:600;\">%s<br></span>" \
-            % self.client.name
-        tool_tip += "<span style=\" font-style:italic;\">%s</span></p>" \
-            % self.client.description
-        tool_tip += "<p></p>"
-        tool_tip += "<p>%s : %s<br>" \
-            % (_translate('client_slot', 'Protocol'),
-               ray.protocol_to_str(self.client.protocol))
-        tool_tip += "%s : %s<br>" \
-            % (_translate('client_slot', 'Executable'),
-               self.client.executable_path)
-        tool_tip += "%s : %s</p>" \
-            % (_translate('client_slot', 'client id'), self.client.client_id)
-        tool_tip += "</body></html>"
+        tool_tip = (
+            '<html><head/><body>'
+            '<p>'
+                '<span style="font-weight:600;">'
+                    f'{self.client.name}<br>'
+                '</span>'
+                '<span style="font-style:italic;">'
+                    f'{self.client.description}'
+                '</span>'
+            '</p>'
+            '<p></p>'
+            '<p>'
+                f"{_translate('client_slot', 'Protocol')} : "
+                f'{self.client.protocol.to_string()}'
+                '<br>'
+                f"{_translate('client_slot', 'Executable')} : "
+                f"{self.client.executable_path}"
+                '<br>'
+                f"{_translate('client_slot', 'client id')} : "
+                f'{self.client.client_id}'
+            '</p>'
+            '</body></html>'
+        )
 
         self.ui.ClientName.setToolTip(tool_tip)
 
         # set icon
         self._icon_on = get_app_icon(self.client.icon, self)
-        self._icon_off = QIcon(self._icon_on.pixmap(32, 32, QIcon.Disabled))
+        self._icon_off = QIcon(self._icon_on.pixmap(32, 32, QIcon.Mode.Disabled))
 
         self._gray_icon(
             bool(self.client.status in (
@@ -325,40 +347,22 @@ class ClientSlot(QFrame):
             self.ui.toolButtonGUI.setVisible(
                 bool(':optional-gui:' in self.client.capabilities))
             self.set_gui_state(self.client.gui_state)
-
-        if self.client.executable_path in ('ray-proxy', 'nsm-proxy'):
-            if is_dark_theme(self):
-                self._icon_visible = QIcon()
-                self._icon_visible.addPixmap(
-                    QPixmap(':scalable/breeze-dark/emblem-symbolic-link'),
-                    QIcon.Normal, QIcon.Off)
-                self._icon_invisible = QIcon()
-                self._icon_invisible.addPixmap(
-                    QPixmap(':scalable/breeze-dark/link'),
-                    QIcon.Normal, QIcon.Off)
-                self._icon_invisible.addPixmap(
-                    QPixmap(':scalable/breeze-dark/disabled/link'),
-                    QIcon.Disabled, QIcon.Off)
+        
+        if (self.client.protocol is ray.Protocol.RAY_HACK
+                and self.client.ray_hack is not None):
+            if self.client.ray_hack.relevant_no_save_level():
+                self.ui.saveButton.setIcon(self._no_save_icon) # type:ignore
             else:
-                self._icon_visible = QIcon()
-                self._icon_visible.addPixmap(
-                    QPixmap(':scalable/breeze/emblem-symbolic-link'),
-                    QIcon.Normal, QIcon.Off)
-                self._icon_invisible = QIcon()
-                self._icon_invisible.addPixmap(
-                    QPixmap(':scalable/breeze/link'), QIcon.Normal, QIcon.Off)
-                self._icon_invisible.addPixmap(
-                    QPixmap(':scalable/breeze/disabled/link'),
-                    QIcon.Disabled, QIcon.Off)
+                self.ui.saveButton.setIcon(self._save_icon) # type:ignore
 
-    def update_status(self, status: int):
+    def update_status(self, status: ray.ClientStatus):
         self.ui.lineEditClientStatus.setText(client_status_string(status))
         self.ui.lineEditClientStatus.setEnabled(
-            status != ray.ClientStatus.STOPPED)
+            status is not ray.ClientStatus.STOPPED)
         self.ui.actionFindBoxesInPatchbay.setEnabled(
             status not in (ray.ClientStatus.STOPPED, ray.ClientStatus.PRECOPY)) 
 
-        ray_hack = bool(self.client.protocol == ray.Protocol.RAY_HACK)
+        ray_hack = bool(self.client.protocol is ray.Protocol.RAY_HACK)
 
         if status in (
                 ray.ClientStatus.LAUNCH,
@@ -379,7 +383,7 @@ class ClientSlot(QFrame):
                 self.ui.startButton.setVisible(False)
                 self.ui.stopButton.setVisible(True)
 
-        elif status == ray.ClientStatus.READY:
+        elif status is ray.ClientStatus.READY:
             self.ui.startButton.setEnabled(False)
             self.ui.stopButton.setEnabled(True)
             self.ui.closeButton.setEnabled(False)
@@ -393,7 +397,7 @@ class ClientSlot(QFrame):
                 self.ui.startButton.setVisible(False)
                 self.ui.stopButton.setVisible(True)
 
-        elif status == ray.ClientStatus.STOPPED:
+        elif status is ray.ClientStatus.STOPPED:
             self.ui.startButton.setEnabled(True)
             self.ui.stopButton.setEnabled(False)
             self.ui.saveButton.setEnabled(False)
@@ -407,14 +411,14 @@ class ClientSlot(QFrame):
                 self.ui.startButton.setVisible(True)
                 self.ui.stopButton.setVisible(False)
 
-            self.ui.saveButton.setIcon(self._save_icon)
-            self.ui.stopButton.setIcon(self._stop_icon)
+            self.ui.saveButton.setIcon(self._save_icon) # type:ignore
+            self.ui.stopButton.setIcon(self._stop_icon) # type:ignore
             self._stop_is_kill = False
 
             if not ray_hack:
                 self.set_gui_state(False)
 
-        elif status == ray.ClientStatus.PRECOPY:
+        elif status is ray.ClientStatus.PRECOPY:
             self.ui.startButton.setEnabled(False)
             self.ui.stopButton.setEnabled(False)
             self.ui.saveButton.setEnabled(False)
@@ -428,16 +432,16 @@ class ClientSlot(QFrame):
                 self.ui.startButton.setVisible(True)
                 self.ui.stopButton.setVisible(False)
 
-            self.ui.saveButton.setIcon(self._save_icon)
-            self.ui.stopButton.setIcon(self._stop_icon)
+            self.ui.saveButton.setIcon(self._save_icon) # type:ignore
+            self.ui.stopButton.setIcon(self._stop_icon) # type:ignore
             self._stop_is_kill = False
 
-        elif status == ray.ClientStatus.COPY:
+        elif status is ray.ClientStatus.COPY:
             self.ui.saveButton.setEnabled(False)
 
     def allow_kill(self):
         self._stop_is_kill = True
-        self.ui.stopButton.setIcon(self._kill_icon)
+        self.ui.stopButton.setIcon(self._kill_icon) # type:ignore
 
     def flash_if_open(self, flash: bool):
         if flash:
@@ -450,31 +454,27 @@ class ClientSlot(QFrame):
         self.ui.toolButtonHack.setChecked(state)
 
     def show_gui_button(self):
-        self.ui.toolButtonGUI.setIcon(self._icon_invisible)
+        self.ui.toolButtonGUI.setIcon(self._icon_invisible) # type:ignore
         self.ui.toolButtonGUI.setVisible(True)
 
     def set_gui_state(self, state: bool):
         if state:
-            self.ui.toolButtonGUI.setIcon(self._icon_visible)
+            self.ui.toolButtonGUI.setIcon(self._icon_visible) # type:ignore
         else:
-            self.ui.toolButtonGUI.setIcon(self._icon_invisible)
+            self.ui.toolButtonGUI.setIcon(self._icon_invisible) # type:ignore
 
         self._gui_state = state
 
     def set_dirty_state(self, dirty: bool):
         self.ui.saveButton.setIcon(
-            self._unsaved_icon if dirty else self._saved_icon)
-
-    def set_no_save_level(self, no_save_level: int):
-        self.ui.saveButton.setIcon(
-            self._no_save_icon if no_save_level else self._save_icon)
+            self._unsaved_icon if dirty else self._saved_icon) # type:ignore
 
     def set_progress(self, progress: float):
         self.ui.lineEditClientStatus.set_progress(progress)
 
-    def set_daemon_options(self, options):
-        has_git = bool(options & ray.Option.HAS_GIT)
-        self.ui.actionReturnToAPreviousState.setVisible(has_git)
+    def set_daemon_options(self, options: ray.Option):
+        self.ui.actionReturnToAPreviousState.setVisible(
+            ray.Option.HAS_GIT in options)
 
     def patchbay_is_shown(self, yesno: bool):
         self.ui.actionFindBoxesInPatchbay.setVisible(yesno)
@@ -484,8 +484,8 @@ class ClientSlot(QFrame):
         event.accept()
         
     def mousePressEvent(self, event: QMouseEvent) -> None:
-        if (event.button() == Qt.LeftButton
-                and self.client.status != ray.ClientStatus.STOPPED
+        if (event.button() == Qt.MouseButton.LeftButton
+                and self.client.status is not ray.ClientStatus.STOPPED
                 and self.client.jack_client_name
                 and self.list_widget_item.isSelected()):
             self.client.session.patchbay_manager.select_client_box(
@@ -495,7 +495,7 @@ class ClientSlot(QFrame):
 
 class ClientItem(QListWidgetItem):
     def __init__(self, parent: 'ListWidgetClients', client_data):
-        QListWidgetItem.__init__(self, parent, QListWidgetItem.UserType + 1)
+        QListWidgetItem.__init__(self, parent, QListWidgetItem.ItemType.UserType + 1)
 
         self.sort_number = 0
         self.widget = ClientSlot(parent, self, client_data)
@@ -524,11 +524,11 @@ class ListWidgetClients(QListWidget):
         if server:
             server.to_daemon(*args)
 
-    @pyqtSlot()
+    @Slot()
     def _launch_favorite(self):
-        template_name, factory = self.sender().data()
+        template_name, factory = self.sender().data() # type:ignore
         self.to_daemon(
-            '/ray/session/add_client_template',
+            r.session.ADD_CLIENT_TEMPLATE,
             int(factory),
             template_name,
             'start',
@@ -543,7 +543,7 @@ class ListWidgetClients(QListWidget):
 
     def remove_client_widget(self, client_id):
         for i in range(self.count()):
-            item: ClientItem = self.item(i)
+            item: ClientItem = self.item(i) # type:ignore
             if item.get_client_id() == client_id:
                 widget = item.widget
                 self.takeItem(i)
@@ -567,8 +567,11 @@ class ListWidgetClients(QListWidget):
             widget = item.widget
             widget.patchbay_is_shown(yesno)
 
+    def item(self, index: int) -> ClientItem:
+        return super().item(index) # type:ignore
+
     def currentItem(self) -> ClientItem:
-        return super().currentItem()
+        return super().currentItem() # type:ignore
 
     def dropEvent(self, event):
         QListWidget.dropEvent(self, event)
@@ -582,7 +585,7 @@ class ListWidgetClients(QListWidget):
 
         server = GuiServerThread.instance()
         if server:
-            server.to_daemon('/ray/session/reorder_clients', *client_ids_list)
+            server.to_daemon(r.session.REORDER_CLIENTS, *client_ids_list)
 
     def mousePressEvent(self, event):
         if not self.itemAt(event.pos()):
@@ -614,8 +617,9 @@ class ListWidgetClients(QListWidget):
                 menu.addMenu(fav_menu)
 
                 menu.addAction(
-                    self.session.main_win.ui.actionAddApplication)
-                menu.addAction(self.session.main_win.ui.actionAddExecutable)
+                    self.session.main_win.ui.actionAddApplication) # type:ignore
+                menu.addAction(
+                    self.session.main_win.ui.actionAddExecutable) # type:ignore
 
                 act_selected = menu.exec(self.mapToGlobal(event.pos()))
             event.accept()
@@ -624,8 +628,8 @@ class ListWidgetClients(QListWidget):
     def resizeEvent(self, event):
         QListWidget.resizeEvent(self, event)
         for i in range(self.count()):
-            item: ClientItem = self.item(i)
-            widget: ClientSlot = self.itemWidget(item)
+            item: ClientItem = self.item(i) # type:ignore
+            widget: ClientSlot = self.itemWidget(item) # type:ignore
             if widget is not None:
                 widget.update_layout()
 
@@ -633,12 +637,12 @@ class ListWidgetClients(QListWidget):
         super().keyPressEvent(event)
         
         # parse patchbay boxes of the selected client 
-        if event.key() in (Qt.Key_Left, Qt.Key_Right):
+        if event.key() in (Qt.Key.Key_Left, Qt.Key.Key_Right):
             client = self.currentItem().widget.client
-            if (client.status != ray.ClientStatus.STOPPED
+            if (client.status is not ray.ClientStatus.STOPPED
                     and client.jack_client_name
                     and self.currentItem().isSelected()
                     and self.session is not None):
                 self.session.patchbay_manager.select_client_box(
                     client.jack_client_name,
-                    previous=bool(event.key() == Qt.Key_Left))
\ No newline at end of file
+                    previous=bool(event.key() == Qt.Key.Key_Left))
\ No newline at end of file
