@@ -1,11 +1,17 @@
-from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QFrame, QMenu, QBoxLayout
-from PyQt5.QtGui import QIcon, QFontMetrics, QContextMenuEvent, QMouseEvent
-from PyQt5.QtCore import QSize, pyqtSignal
 
+# third party imports
+from qtpy.QtWidgets import QListWidget, QListWidgetItem, QFrame, QMenu
+from qtpy.QtGui import QIcon, QFontMetrics, QContextMenuEvent, QMouseEvent
+from qtpy.QtCore import QSize, Signal # type:ignore
+
+# Imports from src/shared
 import ray
+
+# Local imports
 from gui_server_thread import GuiServerThread
 from gui_tools import _translate, split_in_two, get_app_icon
 
+# Import UIs made with Qt-Designer
 import ui.preview_client_slot
 
 
@@ -29,10 +35,10 @@ class ClientSlot(QFrame):
             self._properties_request)
 
         self._menu = QMenu(self)
-        self._menu.addAction(self.ui.actionAddToTheCurrentSession)
-        self._menu.addAction(self.ui.actionProperties)
+        self._menu.addAction(self.ui.actionAddToTheCurrentSession) # type:ignore
+        self._menu.addAction(self.ui.actionProperties) # type:ignore
 
-        self.ui.iconButton.setMenu(self._menu)
+        self.ui.iconButton.setMenu(self._menu) # type:ignore
         self.update_client_data()
         
         self._server_status = ray.ServerStatus.OFF
@@ -45,9 +51,9 @@ class ClientSlot(QFrame):
 
     def _gray_icon(self, gray: bool):
         if gray:
-            self.ui.iconButton.setIcon(self._icon_off)
+            self.ui.iconButton.setIcon(self._icon_off) # type:ignore
         else:
-            self.ui.iconButton.setIcon(self._icon_on)
+            self.ui.iconButton.setIcon(self._icon_on) # type:ignore
 
     def _properties_request(self):
         self._list_widget.properties_request.emit(self.get_client_id())
@@ -59,21 +65,21 @@ class ClientSlot(QFrame):
         self._gray_icon(not launched)
         self.ui.ClientName.setEnabled(launched)
 
-    def server_status_changed(self, server_status:int):
+    def server_status_changed(self, server_status: ray.ServerStatus):
         self.ui.actionAddToTheCurrentSession.setEnabled(
-            server_status == ray.ServerStatus.READY)
+            server_status is ray.ServerStatus.READY)
 
     def get_client_id(self):
         return self.client.client_id
 
     def update_layout(self):
         font = self.ui.ClientName.font()
-        main_size = QFontMetrics(font).width(self.client.prettier_name())
+        main_size = QFontMetrics(font).horizontalAdvance(self.client.prettier_name())
 
         layout_width = self._list_widget.width()
 
         scroll_bar = self._list_widget.verticalScrollBar()
-        if scroll_bar.isVisible():
+        if scroll_bar is not None and scroll_bar.isVisible():
             layout_width -= scroll_bar.width()
 
         max_label_width = layout_width - 50
@@ -91,7 +97,7 @@ class ClientSlot(QFrame):
             if not text:
                 continue
 
-            size = QFontMetrics(font).width(text)
+            size = QFontMetrics(font).horizontalAdvance(text)
             max_size = max(max_size, size)
 
         if max_size <= max_label_width:
@@ -123,7 +129,7 @@ class ClientSlot(QFrame):
         tool_tip += "<p></p>"
         tool_tip += "<p>%s : %s<br>" \
             % (_translate('client_slot', 'Protocol'),
-               ray.protocol_to_str(self.client.protocol))
+               self.client.protocol.to_string())
         tool_tip += "%s : %s<br>" \
             % (_translate('client_slot', 'Executable'),
                self.client.executable_path)
@@ -135,7 +141,7 @@ class ClientSlot(QFrame):
 
         # set icon
         self._icon_on = get_app_icon(self.client.icon, self)
-        self._icon_off = QIcon(self._icon_on.pixmap(32, 32, QIcon.Disabled))
+        self._icon_off = QIcon(self._icon_on.pixmap(32, 32, QIcon.Mode.Disabled))
         self._gray_icon(False)
 
     def contextMenuEvent(self, event: QContextMenuEvent):
@@ -145,7 +151,7 @@ class ClientSlot(QFrame):
 
 class ClientItem(QListWidgetItem):
     def __init__(self, parent: 'ListWidgetPreviewClients', client_data):
-        QListWidgetItem.__init__(self, parent, QListWidgetItem.UserType + 1)
+        QListWidgetItem.__init__(self, parent, QListWidgetItem.ItemType.UserType + 1)
 
         self.sort_number = 0
         self.widget = ClientSlot(parent, self, client_data)
@@ -164,8 +170,8 @@ class ClientItem(QListWidgetItem):
 
 
 class ListWidgetPreviewClients(QListWidget):
-    properties_request = pyqtSignal(str)
-    add_to_session_request = pyqtSignal(str)
+    properties_request = Signal(str)
+    add_to_session_request = Signal(str)
 
     def __init__(self, parent):
         QListWidget.__init__(self, parent)
@@ -179,7 +185,7 @@ class ListWidgetPreviewClients(QListWidget):
         if server:
             server.to_daemon(*args)
 
-    def server_status_changed(self, server_status:int):
+    def server_status_changed(self, server_status: ray.ServerStatus):
         self.server_status = server_status
         for i in range(self.count()):
             item: ClientItem = self.item(i)
@@ -201,6 +207,9 @@ class ListWidgetPreviewClients(QListWidget):
                 del item
                 break
 
+    def item(self, index: int) -> ClientItem:
+        return super().item(index) # type:ignore
+
     def mousePressEvent(self, event: QMouseEvent):
         if not self.itemAt(event.pos()):
             self.setCurrentRow(-1)
@@ -212,7 +221,7 @@ class ListWidgetPreviewClients(QListWidget):
         QListWidget.resizeEvent(self, event)
         for i in range(self.count()):
             item: ClientItem = self.item(i)
-            widget: ClientSlot = self.itemWidget(item)
+            widget: ClientSlot = self.itemWidget(item) # type:ignore
             if widget is not None:
                 widget.update_layout()
 
