@@ -1,99 +1,77 @@
 
+# Imports from standard library
 import json
 from pathlib import Path
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING
 import os
-import sys
+import logging
 
-import ray
-import xdg
-from gui_server_thread import GuiServerThread
-from gui_tools import RS, get_code_root
-
-from jack_renaming_tools import group_belongs_to_client
-from patchbay.base_elements import (Group, GroupPos, PortgroupMem,
-                                    PortMode, BoxLayoutMode, PortType, ToolDisplayed,
-                                    PortTypesViewFlag)
+# Imports from HoustonPatchbay
+from patshared import (
+    GroupPos, PortgroupMem, PortMode, ViewData, Naming,
+    PrettyDiff, TransportWanted)
+from patchbay.bases.elements import CanvasOptimize, CanvasOptimizeIt, ToolDisplayed
+from patchbay.bases.group import Group
 from patchbay import (
     PatchbayManager,
     Callbacker,
     PatchbayToolsWidget,
     CanvasOptionsDialog,
     CanvasMenu,
-    patchcanvas
-)
+    patchcanvas)
+
+# Imports from src/shared
+import ray
+import xdg
+from jack_renaming_tools import group_belongs_to_client
+import osc_paths.ray as r
+
+# Local imports
+from gui_server_thread import GuiServerThread
+from gui_tools import RS, get_code_root
 
 if TYPE_CHECKING:
     from gui_session import Session
     from main_window import MainWindow
 
 
-def convert_group_pos_from_ray_to_patchbay(
-        ray_gpos: ray.GroupPosition) -> GroupPos:
-    gpos = GroupPos()
-    gpos.port_types_view = PortTypesViewFlag(ray_gpos.port_types_view)
-    gpos.group_name = ray_gpos.group_name
-    gpos.null_zone = ray_gpos.null_zone
-    gpos.in_zone = ray_gpos.in_zone
-    gpos.out_zone = ray_gpos.out_zone
-    gpos.null_xy = ray_gpos.null_xy
-    gpos.in_xy = ray_gpos.in_xy
-    gpos.out_xy = ray_gpos.out_xy
-    gpos.flags = ray_gpos.flags
-    
-    for port_mode in (PortMode.INPUT, PortMode.OUTPUT, PortMode.BOTH):
-        layout_mode = ray_gpos.get_layout_mode(port_mode.value)
-        gpos.set_layout_mode(port_mode, BoxLayoutMode(layout_mode))
-
-    gpos.fully_set = ray_gpos.fully_set
-    return gpos
-
-def convert_group_pos_from_patchbay_to_ray(
-        gpos: GroupPos) -> ray.GroupPosition:
-    ray_gpos = ray.GroupPosition.new_from(
-        int(gpos.port_types_view), gpos.group_name,
-        gpos.null_zone, gpos.in_zone, gpos.out_zone,
-        *gpos.null_xy, *gpos.in_xy, *gpos.out_xy,
-        int(gpos.flags), 0)
-    
-    for port_mode, box_layout_mode in gpos.layout_modes.items():
-        ray_gpos.set_layout_mode(port_mode.value, box_layout_mode.value)
-    ray_gpos.fully_set = gpos.fully_set
-
-    return ray_gpos
-
-def convert_portgrp_mem_from_ray_to_patchbay(
-        ray_pgmem: ray.PortGroupMemory) -> PortgroupMem:
-    pgmem = PortgroupMem()
-    pgmem.group_name = ray_pgmem.group_name
-    try:
-        pgmem.port_mode = PortMode(ray_pgmem.port_mode)
-    except ValueError:
-        pgmem.port_mode = PortMode.NULL
-
-    try:
-        pgmem.port_type = PortType(ray_pgmem.port_type)
-    except ValueError:
-        pgmem.port_type = PortType.NULL
-    
-    pgmem.above_metadatas = bool(ray_pgmem.above_metadatas) 
-    # TODO, see if copy is required, probably not
-    pgmem.port_names = ray_pgmem.port_names
-    return pgmem
-
-def convert_portgrp_mem_from_patchbay_to_ray(
-        pgmem: PortgroupMem) -> ray.PortGroupMemory:
-    return ray.PortGroupMemory.new_from(
-        pgmem.group_name, pgmem.port_type.value, pgmem.port_mode.value,
-        int(pgmem.above_metadatas), *pgmem.port_names)
+_logger = logging.getLogger(__name__)
 
 
 class RayPatchbayCallbacker(Callbacker):
     def __init__(self, manager: 'RayPatchbayManager'):
         super().__init__(manager)
         self.mng = manager
+    
+    def group_rename(
+            self, group_id: int, pretty_name: str, save_in_jack: bool):
+        group = self.mng.get_group_from_id(group_id)
+        if group is None:
+            return
+
+        if not group.uuid:
+            save_in_jack = False
+
+        self.mng.send_to_daemon(
+            r.server.patchbay.SAVE_GROUP_CUSTOM_NAME,
+            group.name, pretty_name, group.mdata_pretty_name,
+            int(save_in_jack))
         
-    def _ports_connect(self, group_out_id: int, port_out_id: int,
+    def port_rename(self, group_id: int, port_id: int,
+                     pretty_name: str, save_in_jack: bool):
+        port = self.mng.get_port_from_id(group_id, port_id)
+        if port is None:
+            return
+        
+        if not port.type.is_jack:
+            save_in_jack = False
+        
+        self.mng.send_to_daemon(
+            r.server.patchbay.SAVE_PORT_CUSTOM_NAME,
+            port.full_name_id_free, pretty_name,
+            port.mdata_pretty_name, int(save_in_jack))
+
+    def ports_connect(self, group_out_id: int, port_out_id: int,
                        group_in_id: int, port_in_id: int):
         port_out = self.mng.get_port_from_id(group_out_id, port_out_id)
         port_in = self.mng.get_port_from_id(group_in_id, port_in_id)
@@ -102,19 +80,19 @@ class RayPatchbayCallbacker(Callbacker):
             return
 
         self.mng.send_to_patchbay_daemon(
-            '/ray/patchbay/connect',
+            r.patchbay.CONNECT,
             port_out.full_name, port_in.full_name)
 
-    def _ports_disconnect(self, connection_id: int):
+    def ports_disconnect(self, connection_id: int):
         for connection in self.mng.connections:
             if connection.connection_id == connection_id:
                 self.mng.send_to_patchbay_daemon(
-                    '/ray/patchbay/disconnect',
+                    r.patchbay.DISCONNECT,
                     connection.port_out.full_name,
                     connection.port_in.full_name)
                 break
 
-    def _client_show_gui(self, group_id: int, visible: int):
+    def client_show_gui(self, group_id: int, visible: int):
         group = self.mng.get_group_from_id(group_id)
         if group is None:
             return
@@ -127,7 +105,7 @@ class RayPatchbayCallbacker(Callbacker):
                     client.client_id)
                 break
             
-    def _group_selected(self, group_id: int, splitted_mode: PortMode):
+    def group_selected(self, group_id: int, splitted_mode: PortMode):
         # select client widget matching with the selected box
         group = self.mng.get_group_from_id(group_id)
         if group is None:
@@ -141,7 +119,9 @@ class RayPatchbayCallbacker(Callbacker):
                 break                
 
 
-class RayPatchbayManager(PatchbayManager):    
+class RayPatchbayManager(PatchbayManager):
+    main_win: 'MainWindow'
+    
     def __init__(self, session: 'Session'):
         super().__init__(RS.settings)
         self.session = session
@@ -156,10 +136,7 @@ class RayPatchbayManager(PatchbayManager):
         if not server:
             return
 
-        if server.patchbay_addr is None:
-            return
-
-        server.send(server.patchbay_addr, *args)
+        server.send_patchbay_daemon(*args)
 
     @staticmethod
     def send_to_daemon(*args):
@@ -176,17 +153,14 @@ class RayPatchbayManager(PatchbayManager):
             return {}
 
         try:
-            file = open(file_path, 'r')
+            with open(file_path, 'r') as file:
+                new_dict = json.load(file)
+                assert isinstance(new_dict, dict)
         except IOError:
             return {}
-
-        try:
-            new_dict = json.load(file)
-            assert isinstance(new_dict, dict)
         except ImportError:
             return {}
 
-        file.close()
         return new_dict
     
     def _setup_canvas(self):
@@ -197,7 +171,8 @@ class RayPatchbayManager(PatchbayManager):
         
         app_title = ray.APP_TITLE.lower()
         
-        theme_paths.append(xdg.xdg_data_home() / app_title / SUBMODULE / 'themes')
+        theme_paths.append(
+            xdg.xdg_data_home() / app_title / SUBMODULE / 'themes')
 
         if source_theme_path.exists():
             theme_paths.append(source_theme_path)
@@ -211,7 +186,7 @@ class RayPatchbayManager(PatchbayManager):
             assert isinstance(self.main_win, MainWindow)
 
         self.app_init(self.main_win.ui.graphicsView,
-                      theme_paths,
+                      tuple(theme_paths),
                       callbacker=RayPatchbayCallbacker(self),
                       manual_path=manual_path,
                       default_theme_name='Yellow Boards')
@@ -220,33 +195,44 @@ class RayPatchbayManager(PatchbayManager):
     
     def refresh(self):
         super().refresh()
-        self.send_to_patchbay_daemon('/ray/patchbay/refresh')
+        self.send_to_patchbay_daemon(r.patchbay.REFRESH)
     
     def disannounce(self):
-        self.send_to_patchbay_daemon('/ray/patchbay/gui_disannounce')
+        self.send_to_patchbay_daemon(
+            r.patchbay.GUI_DISANNOUNCE, '')
         super().disannounce()
     
+    def set_views_changed(self):
+        super().set_views_changed()
+        self.send_to_daemon(r.server.patchbay.VIEWS_CHANGED,
+                            json.dumps(self.views.short_data_states()))
+    
+    def save_view_and_port_types_view(self):
+        self.send_to_daemon(
+            r.server.patchbay.VIEW_PTV_CHANGED,
+            self.view_number,
+            self.port_types_view.value)
+    
     def save_group_position(self, gpos: GroupPos):
         super().save_group_position(gpos)
-        ray_gpos = convert_group_pos_from_patchbay_to_ray(gpos)
         self.send_to_daemon(
-            '/ray/server/patchbay/save_group_position', *ray_gpos.spread())
+            r.server.patchbay.SAVE_GROUP_POSITION,
+            self.view_number, *gpos.to_arg_list())
 
-    def save_portgroup_memory(self, portgrp_mem: PortgroupMem):
-        super().save_portgroup_memory(portgrp_mem)
-        ray_pgmem = convert_portgrp_mem_from_patchbay_to_ray(portgrp_mem)
+    def save_portgroup_memory(self, pg_mem: PortgroupMem):
+        super().save_portgroup_memory(pg_mem)
         self.send_to_daemon(
-            '/ray/server/patchbay/save_portgroup',
-            *ray_pgmem.spread())
+            r.server.patchbay.SAVE_PORTGROUP,
+            *pg_mem.to_arg_list())
 
     def change_buffersize(self, buffer_size: int):
         super().change_buffersize(buffer_size)
-        self.send_to_patchbay_daemon('/ray/patchbay/set_buffer_size',
-                                     buffer_size)
+        self.send_to_patchbay_daemon(
+            r.patchbay.SET_BUFFER_SIZE, buffer_size)
 
     def filter_groups(self, text: str, n_select=0) -> int:
-        ''' semi hides groups not matching with text
-            and returns number of matching boxes '''
+        '''semi hides groups not matching with text
+        and returns number of matching boxes '''
         opac_grp_ids = set()
         opac_conn_ids = set()
         
@@ -255,7 +241,7 @@ class RayPatchbayManager(PatchbayManager):
             jack_client_names = list[str]()
             
             for client in self.session.client_list:
-                if (client.status != ray.ClientStatus.STOPPED
+                if (client.status is not ray.ClientStatus.STOPPED
                         and client.client_id in client_ids):
                     jack_client_names.append(client.jack_client_name)
                     if not client.jack_client_name.endswith('.' + client.client_id):
@@ -272,44 +258,21 @@ class RayPatchbayManager(PatchbayManager):
                 else:
                     opac = True
                     opac_grp_ids.add(group.group_id)
-                
-                group.semi_hide(opac)
 
         else:
             for group in self.groups:
-                opac = bool(text.lower() not in group.name.lower()
-                            and text.lower() not in group.display_name.lower())
+                opac = bool(
+                    text.lower() not in group.name.lower()
+                    and text.lower() not in group.graceful_name.lower())
                 if opac:
                     opac_grp_ids.add(group.group_id)
-
-                group.semi_hide(opac)
         
-        for conn in self.connections:
-            opac_conn = bool(
-                conn.port_out.group_id in opac_grp_ids
-                and conn.port_in.group_id in opac_grp_ids)
-            
-            conn.semi_hide(opac_conn)
-            if opac_conn:
-                opac_conn_ids.add(conn.connection_id)
-            
-        for group in self.groups:
-            if group.group_id in opac_grp_ids:
-                group.set_in_front()
-        
-        for conn in self.connections:
-            if conn.connection_id in opac_conn_ids:
-                conn.set_in_front()
-        
-        for conn in self.connections:
-            if conn.connection_id not in opac_conn_ids:
-                conn.set_in_front()
+        patchcanvas.semi_hide_groups(opac_grp_ids)
         
         n_boxes = 0
         
         for group in self.groups:
             if group.group_id not in opac_grp_ids:
-                group.set_in_front()
                 n_grp_boxes = group.get_number_of_boxes()
 
                 if n_select > n_boxes and n_select <= n_boxes + n_grp_boxes:
@@ -318,10 +281,56 @@ class RayPatchbayManager(PatchbayManager):
 
         return n_boxes
 
+    def clear_absents_in_view(self, only_current_ptv=False):
+        if self.view_number is None:
+            return
+        
+        if only_current_ptv:
+            presents = set[str]()
+
+            for group in self.groups:
+                if group.is_in_port_types_view(self.port_types_view):
+                    presents.add(group.current_position.group_name)
+
+            self.views.clear_absents(
+                self.view_number, self.port_types_view, presents)
+
+            out_dict = {'view_num': self.view_number,
+                        'ptv': self.port_types_view.name,
+                        'presents': [g for g in presents]}
+
+            self.send_to_daemon(
+                r.server.patchbay.CLEAR_ABSENTS_IN_VIEW,
+                json.dumps(out_dict))
+            return
+        
+        for ptv in self.views[self.view_number].ptvs.keys():
+            presents = set[str]()
+
+            for group in self.groups:
+                if group.is_in_port_types_view(ptv):
+                    presents.add(group.current_position.group_name)
+
+            self.views.clear_absents(self.view_number, ptv, presents)
+
+            out_dict = {'view_num': self.view_number,
+                        'ptv': ptv.name,
+                        'presents': [g for g in presents]}
+
+            self.send_to_daemon(
+                r.server.patchbay.CLEAR_ABSENTS_IN_VIEW,
+                json.dumps(out_dict))
+
+    def change_view_number(self, new_num: int):
+        ex_view_num = self.view_number
+        super().change_view_number(new_num)
+        self.send_to_daemon(
+            r.server.patchbay.VIEW_NUMBER_CHANGED, ex_view_num, new_num)
+
     def get_corrected_a2j_group_name(self, group_name: str) -> str:
         # fix a2j wrongly substitute '.' with space
         for client in self.session.client_list:
-            if (client.status != ray.ClientStatus.STOPPED
+            if (client.status is not ray.ClientStatus.STOPPED
                     and '.' in client.jack_client_name
                     and (client.jack_client_name.replace('.', ' ', 1)
                             == group_name)):
@@ -339,20 +348,20 @@ class RayPatchbayManager(PatchbayManager):
                 if (client.icon
                         and client.jack_client_name.endswith('.' + client.client_id)
                         and group.name.startswith(client.jack_client_name)):
-                    group.display_name = group.display_name.partition('.')[2]
+                    group.graceful_name = group.graceful_name.partition('.')[2]
                 
                 if client.has_gui:
                     group.set_optional_gui_state(client.gui_state)
                 break
 
     def transport_play_pause(self, play: bool):
-        self.send_to_patchbay_daemon('/ray/patchbay/transport_play', int(play))
+        self.send_to_patchbay_daemon(r.patchbay.TRANSPORT_PLAY, int(play))
     
     def transport_stop(self):
-        self.send_to_patchbay_daemon('/ray/patchbay/transport_stop')
+        self.send_to_patchbay_daemon(r.patchbay.TRANSPORT_STOP)
     
     def transport_relocate(self, frame: int):
-        self.send_to_patchbay_daemon('/ray/patchbay/transport_relocate', frame)
+        self.send_to_patchbay_daemon(r.patchbay.TRANSPORT_RELOCATE, frame)
 
     def change_tools_displayed(self, tools_displayed: ToolDisplayed):
         ex_tool_displayed = self._tools_displayed
@@ -364,28 +373,29 @@ class RayPatchbayManager(PatchbayManager):
 
         if transport_disp != ex_transport_disp:
             if transport_disp & ToolDisplayed.TRANSPORT_CLOCK:
-                int_transp = 2
+                transp_wanted = TransportWanted.FULL
             elif transport_disp & ToolDisplayed.TRANSPORT_PLAY_STOP:
-                int_transp = 1
+                transp_wanted = TransportWanted.STATE_ONLY
             else:
-                int_transp = 0
+                transp_wanted = TransportWanted.NO
 
-            self.send_to_patchbay_daemon('/ray/patchbay/activate_transport',
-                                          int_transp)
+            self.send_to_patchbay_daemon(
+                r.patchbay.ACTIVATE_TRANSPORT, transp_wanted.value)
              
         if (ex_tool_displayed & ToolDisplayed.DSP_LOAD
                 != tools_displayed & ToolDisplayed.DSP_LOAD):
             self.send_to_patchbay_daemon(
-                '/ray/patchbay/activate_dsp_load',
+                r.patchbay.ACTIVATE_DSP_LOAD,
                 int(bool(tools_displayed & ToolDisplayed.DSP_LOAD)))
 
-        #### added functions ####
+    #### added functions ####
     
     def select_client_box(self, jack_client_name: str, previous=False):
         if not jack_client_name:
             self._last_selected_client_name = ''
             self._last_selected_box_n = 0
-            patchcanvas.canvas.scene.clearSelection()
+            if patchcanvas.canvas._scene is not None:
+                patchcanvas.canvas._scene.clearSelection()
             return
         
         box_n = 0
@@ -427,33 +437,58 @@ class RayPatchbayManager(PatchbayManager):
             self._last_selected_box_n = 0
 
     def update_group_position(self, *args):
-        # arguments are these ones delivered from ray.GroupPosition.spread()
-        # Not define them allows easier code modifications.
-        gpos = convert_group_pos_from_ray_to_patchbay(
-            ray.GroupPosition.new_from(*args))
+        view_number = args[0]        
+        gpos = GroupPos.from_arg_list(args[1:])
+        view_data = self.views.get(view_number)
         
-        for gposition in self.group_positions:
-            if (gposition.group_name == gpos.group_name
-                    and gposition.port_types_view == gpos.port_types_view):
-                gposition.eat(gpos)
-                break
-        else:
-            self.group_positions.append(gpos)
+        if view_data is None:
+            view_data = self.views[view_number] = ViewData(
+                gpos.port_types_view)
         
-        if gpos.port_types_view == self.port_types_view:
-            group = self.get_group_from_name(gpos.group_name)
-            if group is not None:
-                group.set_group_position(gpos)
+        ptv_dict = view_data.ptvs.get(gpos.port_types_view)
+        if ptv_dict is None:
+            ptv_dict = view_data.ptvs[gpos.port_types_view] = \
+                dict[str, GroupPos]()
+                
+        ptv_dict[gpos.group_name] = gpos
+
+        # In the past, move a box in a GUI was moving this box in others GUIs.
+        # It takes finally no sense, because the theme can make very different
+        # boxes sizes. We should probably choose that position saver
+        # is the first GUI launched.
+        # Out of this case, if this code is executed, we can assume the view
+        # will be changed very soon and it will update the canvas.
+
+        # if (view_number is self.view_number
+        #         and gpos.port_types_view is self.port_types_view):
+        #     group = self.get_group_from_name(gpos.group_name)
+        #     if group is not None:
+        #         group.set_group_position(
+        #             gpos, redraw=PortMode.NULL, restore=PortMode.BOTH, move_now=False)
 
     def update_portgroup(self, *args):
-        ray_pgmem = ray.PortGroupMemory.new_from(*args)
-        pg_mem = convert_portgrp_mem_from_ray_to_patchbay(ray_pgmem)
+        pg_mem = PortgroupMem.from_arg_list(args)
         self.add_portgroup_memory(pg_mem)
 
         group = self.get_group_from_name(pg_mem.group_name)
         if group is not None:
             group.portgroup_memory_added(pg_mem)
     
+    def views_changed(self, json_dict: str):
+        views_data_json = json_dict
+        self.views.update_from_short_data_states(json.loads(views_data_json))
+
+        if not self.views:
+            self.views.add_view(1)
+
+        if self.view_number not in self.views.keys():
+            self.view_number = self.views.first_view_num()
+
+        self.sg.views_changed.emit()
+        
+        # in the case the port types view or is_white_list is not the same
+        self.change_view(self.view_number)
+    
     def optional_gui_state_changed(self, client_id: str, visible: bool):
         for client in self.session.client_list:
             if client.client_id == client_id:
@@ -462,130 +497,91 @@ class RayPatchbayManager(PatchbayManager):
                         group.set_optional_gui_state(visible)
                 break
     
+    def update_group_pretty_name(self, group_name: str, pretty_name: str):
+        self.custom_names.save_group(group_name, pretty_name, '')
+        group = self.get_group_from_name(group_name)
+        if group is not None:
+            group.rename_in_canvas()
+        self.pretty_diff_checker.client_pretty_name_changed(group_name)
+    
+    def update_port_pretty_name(self, port_name: str, pretty_name: str):
+        self.custom_names.save_port(port_name, pretty_name, '')
+        port = self.get_port_from_name(port_name)
+        if port is not None:
+            port.rename_in_canvas()
+        self.pretty_diff_checker.port_pretty_name_changed(port_name)
+    
+    def change_jack_export_naming(self, naming: Naming):
+        super().change_jack_export_naming(naming)
+        
+        pretty_enable = Naming.CUSTOM in naming
+        self.send_to_daemon(
+            r.server.AUTO_EXPORT_CUSTOM_NAMES, str(pretty_enable))
+    
+    def export_custom_names_to_jack(self):
+        super().export_custom_names_to_jack()
+        self.send_to_patchbay_daemon(r.patchbay.EXPORT_ALL_CUSTOM_NAMES)
+    
+    def import_pretty_names_from_jack(self):
+        super().import_pretty_names_from_jack()
+        self.send_to_patchbay_daemon(r.patchbay.IMPORT_ALL_PRETTY_NAMES)
+    
+    def has_pretty_name_diff(self, pretty_diff_int: int):
+        try:
+            pretty_diff = PrettyDiff(pretty_diff_int)
+        except:
+            _logger.warning(
+                f'received a wrong pretty_diff {pretty_diff_int}')
+            return
+        
+        if self.options_dialog is not None:
+            self.options_dialog.change_pretty_diff(pretty_diff)
+    
+    def pretty_names_locked(self, locked: int):
+        if self.options_dialog is not None:
+            self.options_dialog.set_pretty_names_locked(bool(locked))
+    
     def receive_big_packets(self, state: int):
-        self.optimize_operation(not bool(state))
         if state:
+            self._delayed_orders_timer.stop()
+            
+            with CanvasOptimizeIt(self, CanvasOptimize.VERY_FAST):
+                self._delayed_orders_timeout()
+
+            with CanvasOptimizeIt(self):
+                for group in self.groups:
+                    if group.is_in_port_types_view(self.port_types_view):
+                        group.add_to_canvas()
+                        group.add_all_ports_to_canvas()
+                
+                for connection in self.connections:
+                    connection.add_to_canvas()
+
             self.redraw_all_groups()
+            self.pretty_diff_checker.full_update()
             
     def finish_init(self):
         self.set_main_win(self.session.main_win)
         self._setup_canvas()
         self.set_canvas_menu(CanvasMenu(self))
+        if self.main_win._patchbay_tools is not None:
+            self.set_tools_widget(self.main_win._patchbay_tools)
         self.set_options_dialog(
-            CanvasOptionsDialog(self.main_win, self, RS.settings))
+            CanvasOptionsDialog(self.main_win, self))
+
+    def patchbay_announce(self, jack_running: int, alsa_lib_ok: int,
+                          samplerate: int, buffer_size: int, tcp_url: str):
+        if self.options_dialog is not None:
+            self.options_dialog.enable_alsa_midi(bool(alsa_lib_ok))
         
-    def fast_temp_file_memory(self, temp_path):
-        '''receives a .json file path from daemon with groups positions
-           and portgroups remembered from user.'''
-        canvas_data = self._get_json_contents_from_path(temp_path)
-        if not canvas_data:
-            sys.stderr.write(
-                "RaySession::Failed to load tmp file %s to get canvas positions\n"
-                % temp_path)
-            return
-
-        for key in canvas_data.keys():
-            if key == 'group_positions':
-                for gpos_dict in canvas_data[key]:
-                    gpos = ray.GroupPosition()
-                    gpos.write_from_dict(gpos_dict)
-                    self.update_group_position(*gpos.spread())
-
-            elif key == 'portgroups':
-                for pg_dict in canvas_data[key]:
-                    portgroup_mem = ray.PortGroupMemory()
-                    portgroup_mem.write_from_dict(pg_dict)
-                    self.update_portgroup(*portgroup_mem.spread())
-
-        try:
-            os.remove(temp_path)
-        except:
-            pass
-
-    def fast_temp_file_running(self, temp_path: str):
-        '''receives a .json file path from patchbay daemon with all ports, connections
-           and jack metadatas'''
-            
-        patchbay_data = self._get_json_contents_from_path(temp_path)
-        if not patchbay_data:
-            sys.stderr.write(
-                "RaySession::Failed to load tmp file %s to get JACK ports\n"
-                % temp_path)
-            return
-
-        self.clear_all()
-
-        # optimize_operation allow to not redraw group at each port added.
-        # however, if there is no group position
-        # (i.e. if there is no config at all), it is prefferable to
-        # know where finish the group boxes before to add another one.
-        
-        # very fast operation means that nothing is done in the patchcanvas
-        # everything stays here in this file.
-        if self.group_positions:
-            self.optimize_operation(True)
-            self._set_very_fast_operation(True)
-
-        for key in patchbay_data.keys():
-            if key == 'ports':
-                p: dict[str, Any]
-                for p in patchbay_data[key]:
-                    self.add_port(p.get('name'), p.get('type'),
-                                  p.get('flags'), p.get('uuid'))
-
-            elif key == 'connections':
-                c: dict[str, Any]
-                for c in patchbay_data[key]:
-                    self.add_connection(c.get('port_out_name'),
-                                        c.get('port_in_name'))
-
-        for key in patchbay_data.keys():
-            if key == 'clients':
-                cnu: dict[str, Any]
-                for cnu in patchbay_data[key]:
-                    self.set_group_uuid_from_name(cnu.get('name'), cnu.get('uuid'))
-                break
-
-        for key in patchbay_data.keys():
-            if key == 'metadatas':
-                m: dict[str, Any]
-                for m in patchbay_data[key]:
-                    self.metadata_update(
-                        m.get('uuid'), m.get('key'), m.get('value'))
-                break
-
-        for group in self.groups:
-            group.sort_ports_in_canvas()
-
-        self._set_very_fast_operation(False)
-                
-        for group in self.groups:
-            group.add_all_ports_to_canvas()
-        
-        for conn in self.connections:
-            conn.add_to_canvas()
-
-        self.optimize_operation(False)
-        self.redraw_all_groups()
-
-        try:
-            os.remove(temp_path)
-        except:
-            # if this tmp file can not be removed
-            # this is really not strong.
-            pass
-
-    def patchbay_announce(self, jack_running: int, samplerate: int,
-                          buffer_size: int):
         if self._tools_widget is None:
             return
         
+        if self.main_win is not None:
+            self.main_win.add_patchbay_tools(
+                self._tools_widget, self.canvas_menu)
+
         self._tools_widget.set_samplerate(samplerate)
         self._tools_widget.set_buffer_size(buffer_size)
-        self._tools_widget.set_jack_running(jack_running)
-
-        if self.main_win is not None:
-            if TYPE_CHECKING and not isinstance(self.main_win, MainWindow):
-                return
-            self.main_win.add_patchbay_tools(
-                self._tools_widget, self.canvas_menu)
\ No newline at end of file
+        self._tools_widget.set_jack_running(bool(jack_running))
+        self._tools_widget.set_patchbay_manager(self)
\ No newline at end of file
