@@ -1,34 +1,53 @@
 
-from PyQt5.QtCore import Qt, QDateTime, QDate
-from PyQt5.QtWidgets import QDialogButtonBox, QTreeWidgetItem
+# Imports from standard library
+from enum import IntEnum
+from typing import Optional
 
+# third party imports
+from qtpy.QtCore import Qt, QDateTime, QDate
+from qtpy.QtWidgets import QDialogButtonBox, QTreeWidgetItem
+
+# imports from shared
+import osc_paths.ray as r
+
+# Local imports
 from child_dialogs import ChildDialog
 from gui_tools import _translate, RS
 
+# Import UIs made with Qt-Designer
 import ui.snapshot_name
 import ui.list_snapshots
 import ui.snapshots_info
 
-GROUP_ELEMENT = 0
-GROUP_DAY = 1
-GROUP_MONTH = 2
-GROUP_YEAR = 3
-GROUP_MAIN = 4
+
+class InvalidSnapshot(Exception):
+    def __init__(
+            self, 
+            message='Can not access to this func for an invalid snapshot'):
+        super().__init__(message)
+
+
+class SnGroup(IntEnum):
+    ELEMENT = 0
+    DAY = 1
+    MONTH = 2
+    YEAR = 3
+    MAIN = 4
+
 
 class Snapshot:
     valid = False
     text = ''
-    sub_type = GROUP_ELEMENT
+    sub_type = SnGroup.ELEMENT
     item = None
     before_rewind_to = ''
-    date_time: QDateTime = None
-    rewind_date_time: QDateTime = None
+    rewind_date_time: Optional[QDateTime] = None
     session_name = ""
     label = ''
     rewind_label = ''
     ref = ''
 
-    def __init__(self, date_time):
+    def __init__(self, date_time: Optional[QDateTime]):
         self.date_time = date_time
 
     def __lt__(self, other: 'Snapshot'):
@@ -37,11 +56,16 @@ class Snapshot:
 
         if not self.is_valid():
             return False
+        
+        if self.date_time is None:
+            return False
+        if other.date_time is None:
+            return True
 
         return self.date_time < other.date_time
 
     @staticmethod
-    def new_from_snaptext(snaptext:str):
+    def new_from_snaptext(snaptext: str) -> 'Snapshot':
         time_str_full, line_change, rw_time_str_full_sess = \
             snaptext.partition('\n')
         rw_time_str_full, line_change, session_name = \
@@ -53,8 +77,8 @@ class Snapshot:
         utc_date_time = QDateTime.fromString(time_str, 'yyyy_M_d_h_m_s')
         utc_rw_date_time = QDateTime.fromString(rw_time_str,
                                                 'yyyy_M_d_h_m_s')
-        utc_date_time.setTimeSpec(Qt.OffsetFromUTC)
-        utc_rw_date_time.setTimeSpec(Qt.OffsetFromUTC)
+        utc_date_time.setTimeSpec(Qt.TimeSpec.OffsetFromUTC)
+        utc_rw_date_time.setTimeSpec(Qt.TimeSpec.OffsetFromUTC)
 
         date_time = None
         rw_date_time = None
@@ -73,76 +97,78 @@ class Snapshot:
         snapshot.session_name = session_name
         return snapshot
 
-    def year(self):
+    def year(self) -> int:
+        if self.date_time is None:
+            raise InvalidSnapshot
         return self.date_time.date().year()
 
-    def month(self):
+    def month(self) -> int:
+        if self.date_time is None:
+            raise InvalidSnapshot
         return self.date_time.date().month()
 
-    def day(self):
+    def day(self) -> int:
+        if self.date_time is None:
+            raise InvalidSnapshot
         return self.date_time.date().day()
 
-    def is_valid(self):
-        if not self.date_time:
+    def is_valid(self) -> bool:
+        if self.date_time is None:
             return False
 
         return self.date_time.isValid()
 
-    def is_today(self):
+    def is_today(self) -> bool:
         if not self.date_time:
             return False
 
         return bool(self.date_time.date() == QDate.currentDate())
 
-    def is_yesterday(self):
+    def is_yesterday(self) -> bool:
         if not self.date_time:
             return False
 
         return bool(self.date_time.date() == QDate.currentDate().addDays(-1))
 
-    def can_take(self, other):
+    def can_take(self, other) -> bool:
         return False
 
-    def sort(self):
-        pass
+    def sort(self):...
+    def add(self, other):...
 
-    def add(self, other):
-        pass
-
-    def common_group(self, other: 'Snapshot'):
+    def common_group(self, other: 'Snapshot') -> SnGroup:
         if not (self.is_valid() and other.is_valid()):
-            return GROUP_MAIN
+            return SnGroup.MAIN
 
-        common_group = GROUP_MAIN
+        common_group = SnGroup.MAIN
 
         if self.year() == other.year():
-            common_group = GROUP_YEAR
+            common_group = SnGroup.YEAR
             if self.month() == other.month():
-                common_group = GROUP_MONTH
+                common_group = SnGroup.MONTH
                 if self.day() == other.day():
-                    common_group = GROUP_DAY
+                    common_group = SnGroup.DAY
 
         if common_group <= self.sub_type:
-            return self.sub_type + 1
+            return SnGroup(self.sub_type + 1)
 
         return common_group
 
     def make_item(self, sub_type):
-        if self.is_today():
-            day_string = _translate('snapshots', 'Today')
-        elif self.is_yesterday():
-            day_string = _translate('snapshots', 'Yesterday')
-        elif self.is_valid():
-            day_string = self.date_time.toString('dddd d MMMM yyyy')
-
-        if not self.is_valid():
+        if self.date_time is None:
             display_text = self.text
         else:
+            if self.is_today():
+                day_string = _translate('snapshots', 'Today')
+            elif self.is_yesterday():
+                day_string = _translate('snapshots', 'Yesterday')
+            else:
+                day_string = self.date_time.toString('dddd d MMMM yyyy')
+            
             display_text = _translate('snapshots', "%s at %s") % (
-                            day_string,
-                            self.date_time.toString('HH:mm'))
+                                day_string, self.date_time.toString('HH:mm'))
 
-            if sub_type in (GROUP_YEAR, GROUP_MONTH):
+            if sub_type in (SnGroup.YEAR, SnGroup.MONTH):
                 if not self.is_today() or self.is_yesterday():
                     day_string = self.date_time.toString('dddd d MMMM')
 
@@ -150,7 +176,7 @@ class Snapshot:
                                     day_string,
                                     self.date_time.toString('HH:mm'))
 
-            elif sub_type == GROUP_DAY:
+            elif sub_type is SnGroup.DAY:
                 display_text = _translate('snapshots', "at %s") \
                                  % self.date_time.toString('HH:mm')
 
@@ -175,13 +201,13 @@ class Snapshot:
             display_text += "\n%s" % self.label
 
         item = QTreeWidgetItem([display_text])
-        item.setData(0, Qt.UserRole, self.text)
+        item.setData(0, Qt.ItemDataRole.UserRole, self.text)
 
         return item
 
 
 class SnapGroup(Snapshot):
-    def __init__(self, date_time=None, sub_type=GROUP_MAIN):
+    def __init__(self, date_time=None, sub_type=SnGroup.MAIN):
         Snapshot.__init__(self, date_time)
         self.sub_type = sub_type
         self.valid = True
@@ -191,19 +217,19 @@ class SnapGroup(Snapshot):
         if self.sub_type <= other.sub_type:
             return False
 
-        if self.sub_type == GROUP_MAIN:
+        if self.sub_type is SnGroup.MAIN:
             return True
 
         if self.year() != other.year():
             return False
 
-        if self.sub_type == GROUP_YEAR:
+        if self.sub_type is SnGroup.YEAR:
             return True
 
         if self.month() != other.month():
             return False
 
-        if self.sub_type == GROUP_MONTH:
+        if self.sub_type is SnGroup.MONTH:
             return True
 
         if self.day() != other.day():
@@ -216,7 +242,7 @@ class SnapGroup(Snapshot):
             self.snapshots.append(new_snapshot)
             return
 
-        if self.sub_type <= 1:
+        if self.sub_type in (SnGroup.ELEMENT, SnGroup.DAY):
             # If this group (self) is a day group, just add this snapshot
             self.snapshots.append(new_snapshot)
             return
@@ -248,7 +274,7 @@ class SnapGroup(Snapshot):
             cg_final = 0
             compare_snap = Snapshot(None)
 
-            for cg in (GROUP_DAY, GROUP_MONTH, GROUP_YEAR):
+            for cg in (SnGroup.DAY, SnGroup.MONTH, SnGroup.YEAR):
                 if cg_final:
                     break
 
@@ -312,19 +338,19 @@ class SnapGroup(Snapshot):
         self.snapshots.sort()
         self.snapshots.reverse()
 
-    def make_item(self, sub_type=GROUP_MAIN):
+    def make_item(self, sub_type=SnGroup.MAIN):
         display_text = ''
 
-        if self.sub_type == GROUP_MAIN:
+        if self.sub_type is SnGroup.MAIN:
             return None
 
         if not self.date_time:
             display_text = self.text
-        elif self.sub_type == GROUP_YEAR:
+        elif self.sub_type is SnGroup.YEAR:
             display_text = self.date_time.toString('yyyy')
-        elif self.sub_type == GROUP_MONTH:
+        elif self.sub_type is SnGroup.MONTH:
             display_text = self.date_time.toString('MMMM yyyy')
-        elif self.sub_type == GROUP_DAY:
+        elif self.sub_type is SnGroup.DAY:
             display_text = self.date_time.toString('dddd d MMMM yyyy')
             if self.is_today():
                 display_text = _translate('snapshots', 'Today')
@@ -338,7 +364,7 @@ class SnapGroup(Snapshot):
             item.addChild(sub_item)
 
         # set this group item not selectable
-        item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
+        item.setFlags(item.flags() & ~Qt.ItemFlag.ItemIsSelectable)
 
         return item
 
@@ -389,11 +415,13 @@ class SnapshotsDialog(ChildDialog):
         self.ui.snapshotsList.currentItemChanged.connect(
             self._current_item_changed)
 
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        self.ui.buttonBox.button(
+            QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
 
     def _current_item_changed(self, current, previous):
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
-           bool(current and current.data(0, Qt.UserRole)))
+        self.ui.buttonBox.button(
+            QDialogButtonBox.StandardButton.Ok).setEnabled( # type:ignore
+                bool(current and current.data(0, Qt.ItemDataRole.UserRole)))
 
     def _add_snapshots(self, snaptexts):
         if not snaptexts and not self.main_snap_group.snapshots:
@@ -413,18 +441,21 @@ class SnapshotsDialog(ChildDialog):
         self.ui.snapshotsList.clear()
 
         for snapshot in self.main_snap_group.snapshots:
-            item = snapshot.make_item(GROUP_MAIN)
+            item = snapshot.make_item(SnGroup.MAIN)
             self.ui.snapshotsList.addTopLevelItem(item)
 
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        self.ui.buttonBox.button(
+            QDialogButtonBox.StandardButton.Ok).setEnabled(False) # type:ignore
         self.ui.snapshotsList.clearSelection()
 
     def _no_snapshot_found(self):
         pass
 
-    def get_selected_snapshot(self):
+    def get_selected_snapshot(self) -> Optional[str]:
         item = self.ui.snapshotsList.currentItem()
-        full_str: str = item.data(0, Qt.UserRole)
+        if item is None:
+            return None
+        full_str: str = item.data(0, Qt.ItemDataRole.UserRole)
         snapshot_ref = full_str.partition('\n')[0].partition(':')[0]
 
         return snapshot_ref
@@ -458,7 +489,7 @@ class SessionSnapshotsDialog(SnapshotsDialog):
 
         self.ui.pushButtonSnapshotNow.clicked.connect(self._take_snapshot)
 
-        self.to_daemon('/ray/session/list_snapshots')
+        self.to_daemon(r.session.LIST_SNAPSHOTS)
 
         self.ui.checkBoxAutoSnapshot.stateChanged.connect(
             self._set_auto_snapshot)
@@ -469,13 +500,13 @@ class SessionSnapshotsDialog(SnapshotsDialog):
         if dialog.result():
             snapshot_label = dialog.get_snapshot_name()
             with_save = dialog.save_asked()
-            self.to_daemon('/ray/session/take_snapshot', snapshot_label,
+            self.to_daemon(r.session.TAKE_SNAPSHOT, snapshot_label,
                           int(with_save))
             self.ui.snapshotsList.setVisible(True)
             self.ui.label.setText(self._original_label)
 
     def _set_auto_snapshot(self, bool_snapshot):
-        self.to_daemon('/ray/session/set_auto_snapshot', int(bool_snapshot))
+        self.to_daemon(r.session.SET_AUTO_SNAPSHOT, int(bool_snapshot))
 
     def _no_snapshot_found(self):
         self.ui.label.setText(
@@ -492,7 +523,7 @@ class ClientSnapshotsDialog(SnapshotsDialog):
 
         self.client = client
 
-        self.to_daemon('/ray/client/list_snapshots', client.client_id)
+        self.to_daemon(r.client.LIST_SNAPSHOTS, client.client_id)
         self.resize(0, 0)
 
     def _no_snapshot_found(self):
