@@ -1,25 +1,32 @@
 
+# Imports from standard library
 import os
 import shutil
 from typing import TYPE_CHECKING
 
-from PyQt5.QtWidgets import (QApplication, QFileDialog, QMessageBox,
-                             QDialogButtonBox)
-from PyQt5.QtGui import QIcon
-from PyQt5.QtCore import QProcess, QProcessEnvironment, Qt
+# third party imports
+from qtpy.QtWidgets import (
+    QApplication, QFileDialog, QMessageBox, QDialogButtonBox, QPushButton)
+from qtpy.QtGui import QIcon
+from qtpy.QtCore import QProcess, QProcessEnvironment
 
+# Imports from src/shared
 import ray
+
+# Local imports
 from gui_tools import CommandLineArgs, RS
 from open_session_dialog import OpenSessionDialog
 from child_dialogs import ChildDialog
 
+# Import UIs made with Qt-Designer
+import ui.ardour_convert
+import ui.hydro_rh_nsm
+import ui.ray_to_nsm
+
 if TYPE_CHECKING:
     from gui_session import Session
     from main_window import MainWindow
 
-import ui.ardour_convert
-import ui.hydro_rh_nsm
-import ui.ray_to_nsm
 
 _translate = QApplication.translate
 
@@ -48,7 +55,7 @@ class HydrogenRhNsmDialog(ChildDialog):
         for check_box in self._check_boxes:
             check_box.stateChanged.connect(self._one_state_changed)
 
-        if self.session.server_status != ray.ServerStatus.READY:
+        if self.session.server_status is not ray.ServerStatus.READY:
             self.ui.checkBoxCurrentSession.setEnabled(False)
 
     def _current_session_check(self, state:bool):
@@ -88,14 +95,14 @@ class RayToNsmDialog(ChildDialog):
         
         self.choose_current_session = False
         #self.choose_session
-        choose_button = self.ui.buttonBox.addButton(
+        choose_button: QPushButton = self.ui.buttonBox.addButton(
             _translate('utilities', 'Choose a session'),
-            QDialogButtonBox.AcceptRole)
+            QDialogButtonBox.ButtonRole.AcceptRole) # type:ignore
         choose_button.setIcon(QIcon.fromTheme('folder-open'))
 
-        this_session_button = self.ui.buttonBox.addButton(
+        this_session_button: QPushButton = self.ui.buttonBox.addButton(
             _translate('utilities', 'Convert the current session'),
-            QDialogButtonBox.AcceptRole)
+            QDialogButtonBox.ButtonRole.AcceptRole) # type:ignore
         
         if not self.session.path:
             this_session_button.setVisible(False)
@@ -236,7 +243,7 @@ class UtilityScriptLauncher:
         ardour_session , filter = QFileDialog.getOpenFileName(
             self.main_win,
             _translate('utilities', "Choose an Ardour session to convert..."),
-            os.getenv('HOME'),
+            os.getenv('HOME', ''),
             _translate('utilities', "Ardour sessions (*.ardour)"))
 
         if not ardour_session:
@@ -301,6 +308,10 @@ class UtilityScriptLauncher:
             if not open_dialog.result():
                 return
 
-            args.append(open_dialog.get_selected_session())
+            session_name = open_dialog.get_selected_session()
+            if session_name is None:
+                return
+
+            args.append(session_name)
 
         self._start_process(script_name, terminal_title, *args)
